#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function
import Manyo as mm
import makeHistogram as mH
import makeHistoExtend as mE
import utsusemi.vis.M2Plot as mp
#import numpy as np
#import math
#import os
import sys

from tofHist import * 

ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
else:
    from xml.etree.ElementTree import ElementTree

import getVNRConst #[inamura 180326]

# reserved words for return value in this.
DAT = None
ECA = None
#ecm = None

#########################################
def PickupCurpos(runnumber=124368, meas='A2TH'):
    """
    Pickup params.xml curPos log for VNR
    @param  runnumber    (int) Run Number
    @param  meas         (string) measurement device 
    @retval TH           (double)
    """

    #exID = os.path.join(os.sep,"data",XXX, "exIDrunNo.txt")
    exID = os.path.join(os.sep,getVNRConst.GetDataDir(True), "exIDrunNo.txt") #[inamura 180326]
    f = open( exID)
    lines = f.readlines()
    f.close()
    exno = 0
    curPos = '-9999.9'
    for line in lines:
	words = line.split(",")
	if ( len( words) > 2):
	    if ( int( words[2]) == runnumber):
		exno = int( words[1])
		print("exID: ", exno, " runNo.:", runnumber)
    if ( exno > 0):
	#exID = os.path.join(os.sep,"usr","local","mlf",XXX,"ana","xml","ex"+str(exno).zfill(8),"params.xml")
        exID = os.path.join(os.sep,getVNRConst.GetUserDir(),"ana","xml","ex"+str(exno).zfill(8),"params.xml") #[inamura 180326]
        if os.path.exists(exID):
	    ET = ElementTree()
            dom = ET.parse(exID)
	    exInfo_ele = dom.find("exInfo")
	    rsltlst_ele = exInfo_ele.find("measRsltLst")
	    runno = str(runnumber)
	    for rslt_ele in rsltlst_ele.findall("measRslt"):
	        runNo = rslt_ele.find("runNo").text
		if ( runNo == runno):
		    param_ele = rslt_ele.find("param")
		    loglst_ele = param_ele.find("logLst")
		    if ( isinstance( loglst_ele, type(None)) == False):
			for logs_ele in loglst_ele.findall("logs"):
			    devName = logs_ele.find("devName").text
			    if ( devName == meas):
				curPos = logs_ele.find( ".//curPos").text
				print("Device: ", devName, " Value: ", curPos)
	    if ( curPos == '-9999.9'):
		print("Not found log in params.xml:", exno)
	else:
	    print("Not found params.xml: ", exno) 
    else:
	print("Not found. runNo.:", runnumber)
    TH = float( curPos) 
    return TH

########################################
def GetHistMWPC0d(runStr='123927', initial_bin=10000.0,final_bin=38100.0,delta_bin=0.01, xmin=64, xmax=191, ymin=64, ymax=191):
    """
    Load EventData and convert to histogram for MWPC
    @param  runStr       (String) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  delta_bin    (double) 
    @param  xmin         (int)
    @param  xmax         (int)
    @param  ymin         (int)
    @param  ymax         (int)
    @retval EC           (ElementContainer)
    """

    dioFlag = -1
    pixels = (256,256)

    runArray=runStr.split('+')
    for j in range( len( runArray)):
	runNo = int(runArray[j])
	datalone, T0Coone = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)
	t0c = len( T0Coone)
	if j == 0:
	    datalist = datalone
	    T0Count = T0Coone
	else:
	    for i in range( t0c):
		T0Count[i] += T0Coone[i]
		dataone = datalone[i]
		for y in range( len(dataone) ):
		    for x in range( len(dataone[y]) ):
			if ( dataone[y][x] != []):
			    if ( datalist[i][y][x] != []):
				datalist[i][y][x] = np.concatenate((datalist[i][y][x], dataone[y][x]))
			    else:
				datalist[i][y][x] = dataone[y][x]

    ## datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    runNo = int(runArray[0])
    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0
    bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)
    t0c = len( T0Count)
    for i in range( t0c):
	t0s = T0Count[i]
	event_c = 0
	dataone = datalist[i]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			if ( len( events) > 0):  # 2016-10-17 ##[inamura 180330]
			#if ( events != []):
			    if ( event_c == 0):
				intensity_array = events
			    else:
				intensity_array = np.concatenate((intensity_array, events))
			    event_c += 1
	intensity_a, tof_array = np.histogram( intensity_array, bins=bins )
	err_array = np.sqrt( intensity_a ) / float( t0s)
	intensity_array = intensity_a / float( t0s)
	print("Cell:", event_c)
	hh = mm.HeaderBase()
	hh.Add("RUNNUMBER", str(runNo))
	hh.Add("XAxis", "TOF")
	hh.Add("XLABEL", "TOF")
	hh.Add("YLABEL", "Intensity")
	hh.Add("TotalCount", float(intensity_array.sum()))
	hh.Add("T0Count",t0s)
	if ( t0c > 1):
	    if ( i == 0):
		hh.Add("Label", "mp")
	    else:
		if ( i == 1): 
		    hh.Add("Label", "pp")
		else:
		    if ( i == 2):
			hh.Add("Label", "mm")
		    else:
			hh.Add("Label", "pm")
	a2th = PickupCurpos(runNo, 'A2TH')
	if ( a2th != -9999.9):
	    hh.Add("A2TH", a2th)
        EC  = mm.ElementContainer(hh)
	EC.Add("TOF",tof_array.tolist(), "microsec")
	EC.Add("Intensity", intensity_array.tolist(), "Counts")
	EC.Add("Error", err_array.tolist(), "Counts")
	EC.SetKeys("TOF","Intensity","Error")

    return EC

########################################
def GetHistMWPC0dx(runNo=123927, initial_bin=10000.0,final_bin=38100.0,delta_bin=0.01, xmin=64, xstep=4, ymin=64, ymax=191):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo       (int) Run Number
    @param  initial_bin (double) 
    @param  final_bin   (double) 
    @param  delta_bin   (double) or width_of_bin 
    @param  xmin        (int)
    @param  xstep       (int)
    @param  ymin        (int)
    @param  ymax        (int)
    """

    dioFlag = -1
    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0
    if ( ymin < 0 or ymin > 255):
	ymin = 0
    if ( ymax < 0 or ymax > 255):
	ymax = 255
    if ( xmin < 0 or xmin > 255):
	xmin = 0
    if ( xstep < 1 or xstep > 255):
	xstep = 4

    if ( delta_bin >= 1.0):
	bin = getTofBin_T_Constant(int(initial_bin),int(final_bin),int(delta_bin))
	bins = map(float, bin)
    else:
	bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)
    t0c = len( T0Count)
    #f = open("/usr/local/mlf/VNR/ana/xml/"+str(runNo).zfill(6)+"_x.dat", 'w')
    f = open( os.path.join( getVNRConst.GetUserDir(),"ana","xml",str(runNo).zfill(6)+"_x.dat"), 'w') #[inamura 180403]
    f.write("tof_%d\tref_%d\terr_%d\n" % ( runNo, runNo, runNo))
    for i in range( t0c):
	t0s = T0Count[i]
	dataone = datalist[i]
	hist_n = ( (255 - xmin) / xstep) + 1
	intens = [np.array([])] * hist_n
	for y in range( ymin, ymax + 1):
	    for x0 in range( hist_n):
		for xs in range( 0, xstep):
		    x = xmin + xstep * x0 + xs
		    events = dataone[y][x]
		    #if ( events != []):
                    if ( len( events) > 0):  #[inamura 180403]
			intens[x0] = np.concatenate((intens[x0], events))
	print("Histogram n:", hist_n)
	for x0 in range( hist_n):
	    if ( intens[x0] != []):
		intensity_a, tof_array = np.histogram( intens[x0], bins=bins )
		err_array = np.sqrt( intensity_a )
		tof_a = (tof_array[1:] + tof_array[:-1]) * 0.5
		f.write("\nposition:%d  x = (%d, %d)\n" % ( x0, xmin + xstep * x0, xmin + xstep * (x0 + 1) - 1))
		for j in range( len( tof_a)):
		    f.write("%.12g\t%.16g\t%.16g\n" % (tof_a[j], intensity_a[j], err_array[j]))

    f.close()

########################################
def GetHistMWPC0dTime(runNo=129117, dioFlag=-1, bin_range='10000,38100', delta_bin=0.01, xrange='64,191', yrange='64,191', timerange='0,-1', timestep=-1):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo       (int) Run Number
    @param  dioFlag	(int) 
    @param  bin_range   (string) min, max
    @param  delta_bin   (double) or width_of_bin 
    @param  xrange      (string) min, max
    @param  yrange      (string) min, max
    @param  timerange   (string)
    @param  timestep    (int)   or -1 (not use)
    @retval ECA         (ElementContainer)
    """

    pixels = (256,256)
    words= bin_range.split(",")
    initial_bin = int( words[0])
    if ( len( words) > 1):
	final_bin = int( words[1])
    else:
	final_bin = 80000
    if ( initial_bin > final_bin):
	final_bin = 80000
    words= xrange.split(",")
    xmin = int( words[0])
    if ( len ( words) > 1):
	xmax = int( words[1])
    else:
	xmax = 255
    words= yrange.split(",")
    ymin = int( words[0])
    if ( len ( words) > 1):
	ymax = int( words[1])
    else:
	ymax = 255
    words= timerange.split(",")
    nT0start = int( words[0])
    if ( len ( words) > 1):
	nT0end = int( words[1])
    else:
	nT0end = -1
    if timestep < 1:
	nT0step = -1
    else:
	nT0step = timestep

    datalist, T0Count = mE.getEventDataArrayMulti( runNo, "MWPC", pixels, dioFlag, nT0start, nT0end, nT0step, initial_bin, final_bin)

    if ( delta_bin >= 1.0):
	bin = getTofBin_T_Constant(int(initial_bin),int(final_bin),int(delta_bin))
	bins = map(float, bin)
    else:
	bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)

    a2th = PickupCurpos(runNo, 'A2TH')

    if ( dioFlag == -1 or dioFlag == 0):
	t0c = len( T0Count)
	ECA = []
	for i in range( t0c):
	    t0s = T0Count[i]
	    event_c = 0
	    dataone = datalist[i]
	    for y in range( len(dataone) ):
		if ( y >= ymin and y <= ymax):
		    for x in range( len(dataone[y]) ):
			if ( x >= xmin and x <= xmax):
			    events = dataone[y][x]
			    #if ( events != []):
                            if ( len( events )>0 ):
				if ( event_c == 0):
				    intensity_array = events
				else:
				    intensity_array = np.concatenate((intensity_array, events))
				event_c += 1
	    intensity_a, tof_array = np.histogram( intensity_array, bins=bins )
	    err_array = np.sqrt( intensity_a ) / float( t0s)
	    intensity_array = intensity_a / float( t0s)
	    print("Cell:", event_c)
	    hh = mm.HeaderBase()
	    hh.Add("RUNNUMBER", str(runNo))
	    hh.Add("XAxis", "TOF")
	    hh.Add("XLABEL", "TOF")
	    hh.Add("YLABEL", "Intensity")
	    hh.Add("TotalCount", float(intensity_array.sum()))
	    hh.Add("T0Count",t0s)
	    hh.Add("Label", str(i))
	    if ( a2th != -9999.9):
		hh.Add("A2TH", a2th)
	    EC  = mm.ElementContainer(hh)
	    EC.Add("TOF",tof_array.tolist(), "microsec")
	    EC.Add("Intensity", intensity_array.tolist(), "Counts")
	    EC.Add("Error", err_array.tolist(), "Counts")
	    EC.SetKeys("TOF","Intensity","Error")
	    ECA += [EC]
    else:
	t0l = len( T0Count)
	ECA = []
	for k in range( tol):
	    t0c = len( T0Count[k])
	    ECAA = []
	    for i in range( t0c):
		t0s = T0Count[k][i]
		event_c = 0
		dataone = datalist[k][i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y]) ):
			    if ( x >= xmin and x <= xmax):
				events = dataone[y][x]
				#if ( events != []):
                                if ( len( events) > 0):  #[inamura 180403]
				    if ( event_c == 0):
					intensity_array = events
				    else:
					intensity_array = np.concatenate((intensity_array, events))
				    event_c += 1
		intensity_a, tof_array = np.histogram( intensity_array, bins=bins )
		err_array = np.sqrt( intensity_a ) / float( t0s)
		intensity_array = intensity_a / float( t0s)
		# print "Cell:", event_c
		hh = mm.HeaderBase()
		hh.Add("RUNNUMBER", str(runNo))
		hh.Add("XAxis", "TOF")
		hh.Add("XLABEL", "TOF")
		hh.Add("YLABEL", "Intensity")
		hh.Add("TotalCount", float(intensity_array.sum()))
		hh.Add("T0Count",t0s)
		hh.Add("Label", str(i))
		if ( a2th != -9999.9):
		    hh.Add("A2TH", a2th)
		EC  = mm.ElementContainer(hh)
		EC.Add("TOF",tof_array.tolist(), "microsec")
		EC.Add("Intensity", intensity_array.tolist(), "Counts")
		EC.Add("Error", err_array.tolist(), "Counts")
		EC.SetKeys("TOF","Intensity","Error")
		ECAA += [EC]
	    ECA += [ECAA]
    return ECA

########################################
def GetHistMWPC0dDio(runStr='123924', initial_bin=10000.0,final_bin=38100.0,delta_bin=0.01, xmin=64, xmax=191, ymin=64, ymax=191):
    """
    Load EventData and convert to histogram for MWPC
    @param  runStr       (string) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  delta_bin    (double) 
    @param  xmin         (int)
    @param  xmax         (int)
    @param  ymin         (int)
    @param  ymax         (int)
    @retval ECA          (ElementContainer)
    """

    dioFlag = 2
    pixels = (256,256)

    runArray=runStr.split('+')
    for j in range( len( runArray)):
	runNo = int(runArray[j])
	datalone, T0Coone = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)
	t0c = len( T0Coone)
	if j == 0:
	    datalist = datalone
	    T0Count = T0Coone
	else:
	    for i in range( t0c):
		T0Count[i] += T0Coone[i]
		dataone = datalone[i]
		for y in range( len(dataone) ):
		    for x in range( len(dataone[y]) ):
			if ( dataone[y][x] != []):
			    if ( datalist[i][y][x] != []):
				datalist[i][y][x] = np.concatenate((datalist[i][y][x], dataone[y][x]))
			    else:
				datalist[i][y][x] = dataone[y][x]

    runNo = int(runArray[0])
    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0
    bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)
    t0c = len( T0Count)
    a2th = PickupCurpos(runNo, 'A2TH')
    ECA = []
    for i in range( t0c):
	t0s = T0Count[i]
	dataone = datalist[i]
	event_c = 0
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			if ( len( events) > 0):  # 2016-10-17 ##[inamura 180330]
			#if ( events != []):
			    if ( event_c == 0):
				intensity_array = events
			    else:
				intensity_array = np.concatenate((intensity_array, events))
			    event_c += 1
	intensity_a, tof_array = np.histogram( intensity_array, bins=bins )
	print("T0  :", t0s, " Cell:", event_c)
	err_array = np.sqrt( intensity_a ) / float( t0s)
	intensity_array = intensity_a / float( t0s)
	hh = mm.HeaderBase()
	hh.Add("RUNNUMBER", str(runNo))
	hh.Add("XAxis", "TOF")
	hh.Add("XLABEL", "TOF")
	hh.Add("YLABEL", "Intensity")
	hh.Add("TotalCount", float(intensity_array.sum()))
	hh.Add("T0Count",t0s)
	if ( t0c > 1):
	    if ( i == 0):
		hh.Add("Label", "mp")
	    else:
		if ( i == 1): 
		    hh.Add("Label", "pp")
		else:
		    if ( i == 2):
			hh.Add("Label", "mm")
		    else:
			hh.Add("Label", "pm")
	if ( a2th != -9999.9):
	    hh.Add("A2TH", a2th)
        EC  = mm.ElementContainer(hh)
	EC.Add("TOF",tof_array.tolist(), "microsec")
	EC.Add("Intensity", intensity_array.tolist(), "Counts")
	EC.Add("Error", err_array.tolist(), "Counts")
	EC.SetKeys("TOF","Intensity","Error")
	ECA += [EC]

    return ECA

########################################
def GetHistMWPC0dDiox(runNo=123901, initial_bin=10000.0,final_bin=38100.0,delta_bin=0.01, xmin=64, xstep=4, ymin=64, ymax=191):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo       (int) Run Number
    @param  initial_bin (double) 
    @param  final_bin   (double) 
    @param  delta_bin   (double) or width_of_bin 
    @param  xmin        (int)
    @param  xstep       (int)
    @param  ymin        (int)
    @param  ymax        (int)
    """

    dioFlag = 2
    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0
    if ( ymin < 0 or ymin > 255):
	ymin = 0
    if ( ymax < 0 or ymax > 255):
	ymax = 255
    if ( xmin < 0 or xmin > 255):
	xmin = 0
    if ( xstep < 1 or xstep > 255):
	xstep = 4

    if ( delta_bin >= 1.0):
	bin = getTofBin_T_Constant(int(initial_bin),int(final_bin),int(delta_bin))
	bins = map(float, bin)
    else:
	bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)
    t0c = len( T0Count)
    for i in range( t0c):
	if ( i == 0):
	    #f = open("/usr/local/mlf/VNR/ana/xml/"+str(runNo).zfill(6)+"_mp.dat", 'w')
            f = open( os.path.join( getVNRConst.GetUserDir(),"ana","xml",str(runNo).zfill(6)+"_mp.dat"), 'w') #[inamura 180403]
	else:
	    if ( i == 1):
		#f = open("/usr/local/mlf/VNR/ana/xml/"+str(runNo).zfill(6)+"_pp.dat", 'w')
                f = open( os.path.join( getVNRConst.GetUserDir(),"ana","xml",str(runNo).zfill(6)+"_pp.dat"), 'w') #[inamura 180403]
	    else:
		if ( i == 2):
		    #f = open("/usr/local/mlf/VNR/ana/xml/"+str(runNo).zfill(6)+"_mm.dat", 'w')
                    f = open( os.path.join( getVNRConst.GetUserDir(),"ana","xml",str(runNo).zfill(6)+"_mm.dat"), 'w') #[inamura 180403]
		else:
		    #f = open("/usr/local/mlf/VNR/ana/xml/"+str(runNo).zfill(6)+"_pm.dat", 'w')
                    f = open( os.path.join( getVNRConst.GetUserDir(),"ana","xml",str(runNo).zfill(6)+"_pm.dat"), 'w') #[inamura 180403]
	f.write("tof_%d\tref_%d\terr_%d\n" % ( runNo, runNo, runNo))
	t0s = T0Count[i]
	dataone = datalist[i]
	hist_n = ( (255 - xmin) / xstep) + 1
	intens = [np.array([])] * hist_n
	for y in range( ymin, ymax + 1):
	    for x0 in range( hist_n):
		for xs in range( 0, xstep):
		    x = xmin + xstep * x0 + xs
		    events = dataone[y][x]
		    #if ( events != []):
                    if ( len( events) > 0):  #[inamura 180403]
			intens[x0] = np.concatenate((intens[x0], events))
	print("Histogram n:", hist_n)
	for x0 in range( hist_n):
	    if ( intens[x0] != []):
		intensity_a, tof_array = np.histogram( intens[x0], bins=bins )
		err_array = np.sqrt( intensity_a )
		tof_a = (tof_array[1:] + tof_array[:-1]) * 0.5
		f.write("\nposition:%d  x = (%d, %d)\n" % ( x0, xmin + xstep * x0, xmin + xstep * (x0 + 1) - 1))
		for j in range( len( tof_a)):
		    f.write("%.12g\t%.16g\t%.16g\n" % (tof_a[j], intensity_a[j], err_array[j]))

	f.close()

########################################
def GetHistMWPCDetector(runNo=123260, initial_bin=100.0,final_bin=40000.0,delta_bin=100.0, binType=0, dioFlag=-1):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo    	(int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  delta_bin    (double) 
    @param  binType      (int)
    @param  dioFlag      (int)  
    @retval DAT (ElementContainerMatrix)
    """

    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    DAT = mm.ElementContainerMatrix()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    DAT.AddToHeader("INSTRUMENT", instrument )
    DAT.AddToHeader("RUNNUMBER", str(runNo))
    bank_v = mm.MakeInt4Vector()
    bank_l = mm.MakeInt4Vector()
    for i in range( len(T0Count)):
	bank_v.append( i ) 
	bank_l.append( 256 ) 
    DAT.AddToHeader("BANKIDLIST", bank_v )
    DAT.AddToHeader("BANKSIZELIST", bank_l )
    bank_n = mm.MakeStringVector()
    if i == 0:
	bank_n.append('Bank0')
    else:
	if i == 1:
	    bank_n.append('BankM')
	    bank_n.append('BankP')
	else:
	    bank_n.append('BankMP')
	    bank_n.append('BankPP')
	    bank_n.append('BankMM')
	    bank_n.append('BankPM')
    DAT.AddToHeader("BANKNAMELIST", bank_n )
    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0
    if ( binType == 0):
	if ( delta_bin < 5.0):
	    delta_bin = 10.0
	bins = getTofBin_T_Constant( int(initial_bin),int(final_bin),int(delta_bin))
    else:
	if ( delta_bin < 0.002):
	    delta_bin = 0.01
	bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)

    print("Make ElementContainerMatrix")
    print("binType	>> ", binType)
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("delta bin	>> ", delta_bin, "(", len(bins), "bins)")

    for i in range( len(T0Count)):

	dataone = datalist[i]

	for x in range( 256):
	    eca = mm.ElementContainerArray()
	    eca.AddToHeader("DETID", x + i*256)
	    eca.AddToHeader("BANKID",i)
	    eca.AddToHeader("INDEXINBANK",x)
	    eca.AddToHeader("MASKED",0 )

	    for y in range( len(dataone) ):
		# make histogarm use numpy function
		events = dataone[y][x]
		intensity_array, tof_array = np.histogram( events, bins=bins )
		# make error list from intensity
		err_array = np.sqrt( intensity_array )

		# make ElementContainer
		ec = mm.ElementContainer()
		ec.Add("TOF",tof_array.tolist(), "microsec")
		ec.Add("Intensity", intensity_array.tolist(), "Counts")
		ec.Add("Error", err_array.tolist(), "Counts")
		ec.SetKeys("TOF","Intensity","Error")

		# add ec to eca(ElementContainerArray)
		eca.Add(ec)

	    #if ( (y & 15) == 15):
	#	print '*%d' % (y),
	    # add eca to ecm(ElementContainerMatrix)
	    DAT.Add(eca)

	print("bank:(%d) end." % (i))
    print("runNo.(%d) end." % (runNo))

    return DAT

########################################
def GetHistRPMTDetector(runNo=130494, initial_bin=100.0,final_bin=40000.0,delta_bin=100.0, binType=0, dioFlag=-1):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo    	(int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  delta_bin    (double) 
    @param  binType      (int)
    @param  dioFlag      (int)  
    @retval DAT (ElementContainerMatrix)
    """

    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "RPMT", pixels, dioFlag)

    DAT = mm.ElementContainerMatrix()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    DAT.AddToHeader("INSTRUMENT", instrument )
    DAT.AddToHeader("RUNNUMBER", str(runNo))
    bank_v = mm.MakeInt4Vector()
    bank_l = mm.MakeInt4Vector()
    for i in range( len(T0Count)):
	bank_v.append( i ) 
	bank_l.append( 256 ) 
    DAT.AddToHeader("BANKIDLIST", bank_v )
    DAT.AddToHeader("BANKSIZELIST", bank_l )
    bank_n = mm.MakeStringVector()
    if i == 0:
	bank_n.append('Bank0')
    else:
	if i == 1:
	    bank_n.append('BankM')
	    bank_n.append('BankP')
	else:
	    bank_n.append('BankMP')
	    bank_n.append('BankPP')
	    bank_n.append('BankMM')
	    bank_n.append('BankPM')
    DAT.AddToHeader("BANKNAMELIST", bank_n )
    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0
    if ( binType == 0):
	if ( delta_bin < 5.0):
	    delta_bin = 10.0
	bins = getTofBin_T_Constant( int(initial_bin),int(final_bin),int(delta_bin))
    else:
	if ( delta_bin < 0.002):
	    delta_bin = 0.01
	bins = getTofBin_DT_T_Constant(initial_bin,final_bin,delta_bin)

    print("Make ElementContainerMatrix")
    print("binType	>> ", binType)
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("delta bin	>> ", delta_bin, "(", len(bins), "bins)")

    for i in range( len(T0Count)):

	dataone = datalist[i]

	for x in range( 256):
	    eca = mm.ElementContainerArray()
	    eca.AddToHeader("DETID", x + i*512)
	    eca.AddToHeader("BANKID",i)
	    eca.AddToHeader("INDEXINBANK",x)
	    eca.AddToHeader("MASKED",0 )

	    for y in range( len(dataone) ):
		# make histogarm use numpy function
		events = dataone[y][x]
		intensity_array, tof_array = np.histogram( events, bins=bins )
		# make error list from intensity
		err_array = np.sqrt( intensity_array )

		# make ElementContainer
		ec = mm.ElementContainer()
		ec.Add("TOF",tof_array.tolist(), "microsec")
		ec.Add("Intensity", intensity_array.tolist(), "Counts")
		ec.Add("Error", err_array.tolist(), "Counts")
		ec.SetKeys("TOF","Intensity","Error")

		# add ec to eca(ElementContainerArray)
		eca.Add(ec)

	    #if ( (y & 15) == 15):
	#	print '*%d' % (y),
	    # add eca to ecm(ElementContainerMatrix)
	    DAT.Add(eca)

	print("bank:(%d) end." % (i))
    print("runNo.(%d) end." % (runNo))

    return DAT

########################################
def GetHistMWPCDetectorT(runNo=123260, initial_bin=100.0,final_bin=40000.0, dioFlag=-1, startsec=0, endsec=-1, secwidth=60):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo    	(int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  dioFlag      (int)  
    @param  startsec     (int)  
    @param  endsec       (int)  
    @param  secwidth     (int)  
    @retval DAT (ElementContainerMatrix)
    """

    pixels = (256,256)
    if ( initial_bin < 0.0):
	initial_bin = 0.0
    if ( final_bin <= 0.0):
	final_bin = 80000.0
    if ( initial_bin >= final_bin):
	initila_bin = 0.0

    datalist, T0Count, neutime = mE.getEventDataArrayTime( runNo, "MWPC", pixels, dioFlag, startsec, endsec, initial_bin, final_bin)

    DAT = mm.ElementContainerMatrix()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    DAT.AddToHeader("INSTRUMENT", instrument )
    DAT.AddToHeader("RUNNUMBER", str(runNo))
    bank_v = mm.MakeInt4Vector()
    bank_l = mm.MakeInt4Vector()
    for i in range( len(T0Count)):
	bank_v.append( i ) 
	bank_l.append( 256 ) 
    DAT.AddToHeader("BANKIDLIST", bank_v )
    DAT.AddToHeader("BANKSIZELIST", bank_l )
    bank_n = mm.MakeStringVector()
    if i == 0:
	bank_n.append('Bank0')
    else:
	if i == 1:
	    bank_n.append('BankM')
	    bank_n.append('BankP')
	else:
	    bank_n.append('BankMP')
	    bank_n.append('BankPP')
	    bank_n.append('BankMM')
	    bank_n.append('BankPM')
    DAT.AddToHeader("BANKNAMELIST", bank_n )

    if ( endsec < 0):
	endsec = int( neutime)+ secwidth
    if ( startsec >= endsec):
	startsec = 0
    if ( secwidth < 1):
	secwidth = ( endsec - startsec) / 256 + 1

    print("Make ElementContainerMatrix")
    #print "binType	>> ", binType
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("start sec.	>> ", startsec)
    print("end sec. 	>> ", endsec)
    print("sec.width	>> ", secwidth)
    #print "delta bin	>> ", delta_bin, "(", len(bins), "bins)"

    bins = getTofBin_T_Constant( startsec, endsec, secwidth)

    for i in range( len(T0Count)):

	dataone = datalist[i]

	for x in range( 256):
	    eca = mm.ElementContainerArray()
	    eca.AddToHeader("DETID", x + i*256)
	    eca.AddToHeader("BANKID",i)
	    eca.AddToHeader("INDEXINBANK",x)
	    eca.AddToHeader("MASKED",0 )

	    for y in range( len(dataone) ):
		# make histogarm use numpy function
		events = dataone[y][x]
		intensity_array, tof_array = np.histogram( events, bins=bins )
		# make error list from intensity
		err_array = np.sqrt( intensity_array )

		# make ElementContainer
		ec = mm.ElementContainer()
		#ec.Add("TOF",tof_array.tolist(), "second")
		ec.Add("SEC",tof_array.tolist(), "second")
		ec.Add("Intensity", intensity_array.tolist(), "Counts")
		ec.Add("Error", err_array.tolist(), "Counts")
		#ec.SetKeys("TOF","Intensity","Error")
		ec.SetKeys("SEC","Intensity","Error")

		# add ec to eca(ElementContainerArray)
		eca.Add(ec)

	    #if ( (y & 15) == 15):
	#	print '*%d' % (y),
	    # add eca to ecm(ElementContainerMatrix)
	    DAT.Add(eca)

	print("bank:(%d) end." % (i))
    print("runNo.(%d) end." % (runNo))

    return DAT

########################################
def GetHistMWPCTOF(runNo=125523, initial_bin=10000,final_bin=38100,xmin=32, xmax=224, ymin=32, ymax=224, bank=-1):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (int) 
    @param  final_bin    (int) 
    @param  xmin         (int)
    @param  xmax         (int)
    @param  ymin         (int)
    @param  ymax         (int)
    @param  bank         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    if ( bank < 0):
	dioFlag = -1
	bank = 0
    else:
	dioFlag = 1
 
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    #a2th = PickupCurpos(runNo, 'A2TH')
    #if ( a2th != -9999.9):
    #	angle = float( a2th) / 2.0
    #else:
    #	angle = 0.3

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("dioFlag  	>> ", dioFlag)
    # print "bank    	>> ", bank

    #pix_per = 0.5

    t0c = len( T0Count)
    tstep = 158
    bin_z = [ float( tstep * j) for j in range(257)]

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(256)] for k in range(256)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    for k in range( eventn):
				tof = events[k]
				if ( tof >= initial_bin and tof <= final_bin):
				    iz = int( tof / tstep)
				    data[iz][x].append( tof)
	bin_x = [ float( j) for j in range(257)] 
	for z in range(256):
            int_list = data[z]
	    int_list_at_px = [ float( len( x)) for x in int_list ]
	    err_list_at_px = [ math.sqrt(e) for e in int_list_at_px ]
	    ec = mm.ElementContainer()
	    ec.Add( "X", bin_x)
	    ec.Add( "Intensity", int_list_at_px )
	    ec.Add( "Error", err_list_at_px )
	    ec.SetKeys("X","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_z[z] )
	    xr_v.append( bin_z[z + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'TOF (microsec)')
	p.SetYLabel( 'Detector PositionX')
	p.SetXLog(True)

    # import Cmm
    # Cmm.SaveDataToDump( ECA, "/home/vnradmin/", "sampledata")
    return ECA

########################################
def GetHistMWPCX(runNo=125523, initial_bin=10000.0,final_bin=38100.0,angle=-1.0, xord=-1, yord=-1, dioFlag=-1, bank=0):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  angle        (double) 
    @param  xord         (int)
    @param  yord         (int)
    @param  dioFlag      (int)
    @param  bank         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    a2th = PickupCurpos(runNo, 'A2TH')
    if ( angle < 0.0):
	if ( a2th != -9999.9):
	    angle = float( a2th) / 2.0
	else:
	    angle = 0.3
    if ( dioFlag == -1):
	bank = 0

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("angle    	>> ", angle)
    print("dioFlag  	>> ", dioFlag)
    print("bank    	>> ", bank)

    # xmin,max ymin,max constant
    xmin = 32
    xmax = 224
    ymin = 32
    ymax = 224
    pix_per = 0.5

    t0c = len( T0Count)
    if ( xord == -1 and yord == -1):
	nmax = 0
	for i in range( t0c):
	    if ( i == bank):
		dataone = datalist[i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y])):
			    if ( x >= xmin and x <= xmax):
				n = len(dataone[y][x])
				n += len(dataone[y-1][x-1])
				n += len(dataone[y-1][x])
				n += len(dataone[y][x-1])
				n += len(dataone[y+1][x])
				n += len(dataone[y][x+1])
				n += len(dataone[y+1][x+1])
				n += len(dataone[y+1][x-1])
				n += len(dataone[y-1][x+1])
				if ( n > nmax):
				    nmax = n
				    xord = x
				    yord = y
	print("Auto Origin set")
    print("x origin   	>> ", xord)
    print("y origin 	>> ", yord)

    QC = 2.0 * math.pi * (18.0e7/3955.6)

    qxmax = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(xmax - xord)/2500.0)) - math.cos(math.pi * angle/180.0)) / initial_bin
    qxmin = qxmax
    xvar = [ xmin, xmax]
    yvar = [ ymin, ymax]
    bins = [initial_bin, final_bin]
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0))  - math.cos(math.pi * angle/180.0)) / bin
		if ( qxmax < QxC):
		    qxmax = QxC
		if ( qxmin > QxC):
		    qxmin = QxC
    if ( qxmax < 0.0):
	qxmax = 0.0

    qx_step = ( qxmax - qxmin) / 256.0
    bin_x = [ (qxmin + qx_step * float( j)) for j in range(257)] 

    qzmax = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * angle/180.0 + math.atan(pix_per * float( xmax - xord)/2500.0))) / initial_bin
    qzmin = qzmax
    for x in xvar:
	for bin in bins:
	    QzC = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * angle/180.0 + math.atan(pix_per * float( x - xord)/2500.0))) / bin
	    if ( qzmax < QzC):
		qzmax = QzC
	    if ( qzmin > QzC):
		qzmin = QzC
    if ( qzmin < 5e-2):
	qzmin = 5e-2
	if ( qzmax < qzmin):
	    qzmax = qzmin + 1
    qmin = math.log( qzmin)
    qmax = math.log( qzmax)
    qstep = ( qmax - qmin ) / 256.0
    bin_z = [ math.exp( qmin + qstep * float( j)) for j in range(257)]
    # qz_step = (qzmax - qzmin) / 256.0
    # bin_z = [ (qzmin + qz_step * float( j)) for j in range(257)]

    print("Qx min  	>> ", qxmin)
    print("Qx max  	>> ", qxmax)
    print("Qz min  	>> ", qzmin)
    print("Qz max  	>> ", qzmax)

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(256)] for k in range(256)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    # QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0))
			    QzC = QC * ( math.sin(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) + math.sin(math.pi * angle/180.0))
			    for k in range( eventn):
				tof = float( events[k])
				if ( tof >= initial_bin and tof <= final_bin):
				    iz = 0
				    Qz = QzC / tof
				    if ( Qz >= qzmin):
					while 1:
					    if ( Qz < bin_z[iz+1]): 
						break
					    iz += 1
					    if iz >= 256:
						break
					if ( iz < 256):
					    data[iz][x].append( tof)
	bin_x = [ float( j) for j in range(257)] 
	for z in range(256):
            int_list = data[z]
	    int_list_at_px = [ float( len( x)) for x in int_list ]
	    err_list_at_px = [ math.sqrt(e) for e in int_list_at_px ]
	    ec = mm.ElementContainer()
	    ec.Add( "X", bin_x)
	    ec.Add( "Intensity", int_list_at_px )
	    ec.Add( "Error", err_list_at_px )
	    ec.SetKeys("X","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_z[z] )
	    xr_v.append( bin_z[z + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'qz(nm^(-1))')
	p.SetYLabel( 'Detector Position')
	p.SetXLog(True)

    # import Cmm
    # Cmm.SaveDataToDump( ECA, "/home/vnradmin/", "sampledata")
    return ECA

########################################
def GetHistMWPCX2(runNo=125523, initial_bin=10000.0,final_bin=38100.0,angle=0.35, angle2=0.0, xord=95, yord=128):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  angle        (double) 
    @param  angle2       (double) 
    @param  xord         (int)
    @param  yord         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    dioFlag = -1
    bank = 0
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    #a2th = PickupCurpos(runNo, 'A2TH')
    #if ( angle < 0.0):
    #	if ( a2th != -9999.9):
    #	    angle = float( a2th) / 2.0
    #	else:
    #	    angle = 0.3
    #if ( dioFlag == -1):
    #	bank = 0

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("angle    	>> ", angle)
    print("angle2   	>> ", angle2)
    #print "dioFlag  	>> ", dioFlag
    #print "bank    	>> ", bank

    # xmin,max ymin,max constant
    xmin = 8
    xmax = 248
    ymin = 32
    ymax = 224
    pix_per = 0.5

    t0c = len( T0Count)
    if ( xord == -1 and yord == -1):
	nmax = 0
	for i in range( t0c):
	    if ( i == bank):
		dataone = datalist[i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y])):
			    if ( x >= xmin and x <= xmax):
				n = len(dataone[y][x])
				n += len(dataone[y-1][x-1])
				n += len(dataone[y-1][x])
				n += len(dataone[y][x-1])
				n += len(dataone[y+1][x])
				n += len(dataone[y][x+1])
				n += len(dataone[y+1][x+1])
				n += len(dataone[y+1][x-1])
				n += len(dataone[y-1][x+1])
				if ( n > nmax):
				    nmax = n
				    xord = x
				    yord = y
	print("Auto Origin set")
    print("x origin   	>> ", xord)
    print("y origin 	>> ", yord)

    QC = 2.0 * math.pi * (18.0e7/3955.6)

    qxmax = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(xmax - xord)/2500.0)) - math.cos(math.pi * angle/180.0)) / initial_bin
    qxmin = qxmax
    xvar = [ xmin, xmax]
    yvar = [ ymin, ymax]
    bins = [initial_bin, final_bin]
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QxC = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0))  - math.cos(math.pi * angle/180.0)) / bin
		if ( qxmax < QxC):
		    qxmax = QxC
		if ( qxmin > QxC):
		    qxmin = QxC
    if ( qxmax < 0.0):
	qxmax = 0.0

    qx_step = ( qxmax - qxmin) / 256.0
    bin_x = [ (qxmin + qx_step * float( j)) for j in range(257)] 

    qzmax = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float( xmax - xord)/2500.0))) / initial_bin
    qzmin = qzmax
    for x in xvar:
	for bin in bins:
	    QzC = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float( x - xord)/2500.0))) / bin
	    if ( qzmax < QzC):
		qzmax = QzC
	    if ( qzmin > QzC):
		qzmin = QzC
    if ( qzmin < 5e-2):
	qzmin = 5e-2
	if ( qzmax < qzmin):
	    qzmax = qzmin + 1
    qmin = math.log( qzmin)
    qmax = math.log( qzmax)
    qstep = ( qmax - qmin ) / 256.0
    bin_z = [ math.exp( qmin + qstep * float( j)) for j in range(257)]
    # qz_step = (qzmax - qzmin) / 256.0
    # bin_z = [ (qzmin + qz_step * float( j)) for j in range(257)]

    print("Qx min  	>> ", qxmin)
    print("Qx max  	>> ", qxmax)
    print("Qz min  	>> ", qzmin)
    print("Qz max  	>> ", qzmax)

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(256)] for k in range(256)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    # QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0))
			    QzC = QC * ( math.sin(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) + math.sin(math.pi * angle/180.0))
			    for k in range( eventn):
				tof = float( events[k])
				if ( tof >= initial_bin and tof <= final_bin):
				    iz = 0
				    Qz = QzC / tof
				    if ( Qz >= qzmin):
					while 1:
					    if ( Qz < bin_z[iz+1]): 
						break
					    iz += 1
					    if iz >= 256:
						break
					if ( iz < 256):
					    data[iz][x].append( tof)
	bin_x = [ float( j) for j in range(257)] 
	for z in range(256):
            int_list = data[z]
	    int_list_at_px = [ float( len( x)) for x in int_list ]
	    err_list_at_px = [ math.sqrt(e) for e in int_list_at_px ]
	    ec = mm.ElementContainer()
	    ec.Add( "X", bin_x)
	    ec.Add( "Intensity", int_list_at_px )
	    ec.Add( "Error", err_list_at_px )
	    ec.SetKeys("X","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_z[z] )
	    xr_v.append( bin_z[z + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'qz(nm^(-1))')
	p.SetYLabel( 'Detector Position')
	p.SetXLog(True)

    # import Cmm
    # Cmm.SaveDataToDump( ECA, "/home/vnradmin/", "sampledata")
    return ECA

########################################
def GetHistMWPCQy(runNo=125523, initial_bin=10000.0,final_bin=38100.0,angle=-1.0, xord=-1, yord=-1, dioFlag=-1, bank=0):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  angle        (double) 
    @param  xord         (int)
    @param  yord         (int)
    @param  dioFlag      (int)
    @param  bank         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    a2th = PickupCurpos(runNo, 'A2TH')
    if ( angle < 0.0):
	if ( a2th != -9999.9):
	    angle = float( a2th) / 2.0
	else:
	    angle = 0.3
    if ( dioFlag == -1):
	bank = 0

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("angle    	>> ", angle)
    print("dioFlag  	>> ", dioFlag)
    print("bank    	>> ", bank)

    # xmin,max ymin,max constant
    xmin = 32
    xmax = 224
    ymin = 32
    ymax = 224
    pix_per = 0.5

    t0c = len( T0Count)
    if ( xord == -1 and yord == -1):
	nmax = 0
	for i in range( t0c):
	    if ( i == bank):
		dataone = datalist[i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y])):
			    if ( x >= xmin and x <= xmax):
				n = len(dataone[y][x])
				n += len(dataone[y-1][x-1])
				n += len(dataone[y-1][x])
				n += len(dataone[y][x-1])
				n += len(dataone[y+1][x])
				n += len(dataone[y][x+1])
				n += len(dataone[y+1][x+1])
				n += len(dataone[y+1][x-1])
				n += len(dataone[y-1][x+1])
				if ( n > nmax):
				    nmax = n
				    xord = x
				    yord = y
	print("Auto Origin set")
    print("x origin   	>> ", xord)
    print("y origin 	>> ", yord)

    QC = 2.0 * math.pi * (18.0e7/3955.6)

    qxmax = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(xmax - xord)/2500.0)) * math.cos(math.atan(pix_per * float(ymax - yord)/2500.0)) - math.cos(math.pi * angle/180.0)) / initial_bin
    qxmin = qxmax
    xvar = [ xmin, xmax]
    yvar = [ ymin, ymax]
    bins = [initial_bin, final_bin]
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0)) / bin
		if ( qxmax < QxC):
		    qxmax = QxC
		if ( qxmin > QxC):
		    qxmin = QxC
    if ( qxmax < 0.0):
	qxmax = 0.0

    qx_step = ( qxmax - qxmin) / 256.0
    bin_x = [ (qxmin + qx_step * float( j)) for j in range(257)] 

    qymax = QC *  math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(xmin - xord)/2500.0)) * math.sin(math.atan(pix_per * float(ymin - yord)/2500.0)) / initial_bin
    qymin = qymax
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QyC = QC *  math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0)) / bin
		if ( qymax < QyC):
		    qymax = QyC
		if ( qymin > QyC):
		    qymin = QyC
    qy_step = (qymax - qymin) / 256.0
    bin_y = [ (qymin + qy_step * float( j)) for j in range(257)]

    qzmax = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * angle/180.0 + math.atan(pix_per * float( xmax - xord)/2500.0))) / initial_bin
    qzmin = qzmax
    for x in xvar:
	for bin in bins:
	    QzC = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * angle/180.0 + math.atan(pix_per * float( x - xord)/2500.0))) / bin
	    if ( qzmax < QzC):
		qzmax = QzC
	    if ( qzmin > QzC):
		qzmin = QzC
    qz_step = (qzmax - qzmin) / 256.0
    bin_z = [ (qzmin + qz_step * float( j)) for j in range(257)]

    print("Qx min  	>> ", qxmin)
    print("Qx max  	>> ", qxmax)
    print("Qy min  	>> ", qymin)
    print("Qy max  	>> ", qymax)
    print("Qz min  	>> ", qzmin)
    print("Qz max  	>> ", qzmax)

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(256)] for k in range(256)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0))
			    QyC = QC *  math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0))
			    QzC = QC * ( math.sin(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) + math.sin(math.pi * angle/180.0))
			    for k in range( eventn):
				tof = float( events[k])
				if ( tof >= initial_bin and tof <= final_bin):
				    ix = 0
				    Qx = QxC / tof
				    while 1:
					if ( Qx < bin_x[ix+1]): 
					    break
					ix += 1
					if ix >= 256:
					     break
				    if ( ix < 256):
					iz = 0
					Qz = QzC / tof
					while 1:
					    if ( Qz < bin_z[iz+1]): 
						break
					    iz += 1
					    if iz >= 256:
						break
					if ( iz < 256):
					    data[ix][iz].append( QyC / tof)
	for x in range(256):
            int_list = data[x]
	    int_list_at_py = [ float( len( y)) for y in int_list ]
	    err_list_at_py = [ math.sqrt(e) for e in int_list_at_py ]
	    ec = mm.ElementContainer()
	    ec.Add( "Z", bin_z)
	    ec.Add( "Intensity", int_list_at_py )
	    ec.Add( "Error", err_list_at_py )
	    ec.SetKeys("Z","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_x[x] )
	    xr_v.append( bin_x[x + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'qx(nm^(-1))')
	p.SetYLabel( 'qz(nm^(-1))')

    return ECA

########################################
def GetHistMWPCQy2(runNo=125523, initial_bin=10000.0,final_bin=38100.0,angle=0.35, angle2=0.0, xord=95, yord=128):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  angle        (double) 
    @param  angle2        (double) 
    @param  xord         (int)
    @param  yord         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    dioFlag = -1
    bank = 0
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    #if ( dioFlag == -1):
    #	bank = 0

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("angle    	>> ", angle)
    print("angle2    	>> ", angle2)
    #print "dioFlag  	>> ", dioFlag
    #print "bank    	>> ", bank

    # xmin,max ymin,max constant
    xmin = 8
    xmax = 248
    ymin = 32
    ymax = 224
    pix_per = 0.5

    t0c = len( T0Count)
    if ( xord == -1 and yord == -1):
	nmax = 0
	for i in range( t0c):
	    if ( i == bank):
		dataone = datalist[i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y])):
			    if ( x >= xmin and x <= xmax):
				n = len(dataone[y][x])
				n += len(dataone[y-1][x-1])
				n += len(dataone[y-1][x])
				n += len(dataone[y][x-1])
				n += len(dataone[y+1][x])
				n += len(dataone[y][x+1])
				n += len(dataone[y+1][x+1])
				n += len(dataone[y+1][x-1])
				n += len(dataone[y-1][x+1])
				if ( n > nmax):
				    nmax = n
				    xord = x
				    yord = y
	print("Auto Origin set")
    print("x origin   	>> ", xord)
    print("y origin 	>> ", yord)

    QC = 2.0 * math.pi * (18.0e7/3955.6)

    qxmax = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(xmax - xord)/2500.0)) * math.cos(math.atan(pix_per * float(ymax - yord)/2500.0)) - math.cos(math.pi * angle/180.0)) / initial_bin
    qxmin = qxmax
    xvar = [ xmin, xmax]
    yvar = [ ymin, ymax]
    bins = [initial_bin, final_bin]
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QxC = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0)) / bin
		if ( qxmax < QxC):
		    qxmax = QxC
		if ( qxmin > QxC):
		    qxmin = QxC
    if ( qxmax < 0.0):
	qxmax = 0.0

    qx_step = ( qxmax - qxmin) / 256.0
    bin_x = [ (qxmin + qx_step * float( j)) for j in range(257)] 

    qymax = QC *  math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(xmin - xord)/2500.0)) * math.sin(math.atan(pix_per * float(ymin - yord)/2500.0)) / initial_bin
    qymin = qymax
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QyC = QC *  math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0)) / bin
		if ( qymax < QyC):
		    qymax = QyC
		if ( qymin > QyC):
		    qymin = QyC
    qy_step = (qymax - qymin) / 256.0
    bin_y = [ (qymin + qy_step * float( j)) for j in range(257)]

    qzmax = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float( xmax - xord)/2500.0))) / initial_bin
    qzmin = qzmax
    for x in xvar:
	for bin in bins:
	    QzC = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float( x - xord)/2500.0))) / bin
	    if ( qzmax < QzC):
		qzmax = QzC
	    if ( qzmin > QzC):
		qzmin = QzC
    qz_step = (qzmax - qzmin) / 256.0
    bin_z = [ (qzmin + qz_step * float( j)) for j in range(257)]

    print("Qx min  	>> ", qxmin)
    print("Qx max  	>> ", qxmax)
    print("Qy min  	>> ", qymin)
    print("Qy max  	>> ", qymax)
    print("Qz min  	>> ", qzmin)
    print("Qz max  	>> ", qzmax)

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(256)] for k in range(256)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    QxC = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0))
			    QyC = QC *  math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0))
			    QzC = QC * ( math.sin(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) + math.sin(math.pi * angle/180.0))
			    for k in range( eventn):
				tof = float( events[k])
				if ( tof >= initial_bin and tof <= final_bin):
				    ix = 0
				    Qx = QxC / tof
				    while 1:
					if ( Qx < bin_x[ix+1]): 
					    break
					ix += 1
					if ix >= 256:
					     break
				    if ( ix < 256):
					iz = 0
					Qz = QzC / tof
					while 1:
					    if ( Qz < bin_z[iz+1]): 
						break
					    iz += 1
					    if iz >= 256:
						break
					if ( iz < 256):
					    data[ix][iz].append( QyC / tof)
	for x in range(256):
            int_list = data[x]
	    int_list_at_py = [ float( len( y)) for y in int_list ]
	    err_list_at_py = [ math.sqrt(e) for e in int_list_at_py ]
	    ec = mm.ElementContainer()
	    ec.Add( "Z", bin_z)
	    ec.Add( "Intensity", int_list_at_py )
	    ec.Add( "Error", err_list_at_py )
	    ec.SetKeys("Z","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_x[x] )
	    xr_v.append( bin_x[x + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'qx(nm^(-1))')
	p.SetYLabel( 'qz(nm^(-1))')

    return ECA

########################################
def GetHistMWPCQyoff(runNo=128215, initial_bin=10000.0,final_bin=38100.0,angle=2.0, angle2=0.0, xord=79, yord=128):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  angle        (double) 
    @param  angle2        (double) 
    @param  xord         (int)
    @param  yord         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    dioFlag = -1
    bank = 0
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    #if ( dioFlag == -1):
    #	bank = 0

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("angle    	>> ", angle)
    print("angle2    	>> ", angle2)
    #print "dioFlag  	>> ", dioFlag
    #print "bank    	>> ", bank

    # xmin,max ymin,max constant
    xmin = 8
    xmax = 248
    ymin = 32
    ymax = 224
    pix_per = 0.5

    t0c = len( T0Count)
    if ( xord == -1 and yord == -1):
	nmax = 0
	for i in range( t0c):
	    if ( i == bank):
		dataone = datalist[i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y])):
			    if ( x >= xmin and x <= xmax):
				n = len(dataone[y][x])
				n += len(dataone[y-1][x-1])
				n += len(dataone[y-1][x])
				n += len(dataone[y][x-1])
				n += len(dataone[y+1][x])
				n += len(dataone[y][x+1])
				n += len(dataone[y+1][x+1])
				n += len(dataone[y+1][x-1])
				n += len(dataone[y-1][x+1])
				if ( n > nmax):
				    nmax = n
				    xord = x
				    yord = y
	print("Auto Origin set")
    print("x origin   	>> ", xord)
    print("y origin 	>> ", yord)

    QC = 2.0 * math.pi * (18.0e7/3955.6)

    qxmax = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(xmax - xord)/2500.0)) - math.cos(math.pi * angle/180.0)) / initial_bin
    qxmin = qxmax
    xvar = [ xmin, xmax]
    yvar = [ ymin, ymax]
    bins = [initial_bin, final_bin]
    for x in xvar:
#	for y in yvar:
	    for bin in bins:
		QxC = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0))  - math.cos(math.pi * angle/180.0)) / bin
		if ( qxmax < QxC):
		    qxmax = QxC
		if ( qxmin > QxC):
		    qxmin = QxC
#    if ( qxmax < 0.0):
#	qxmax = 0.0

#    qxmin = -0.01074
#    qxmax = -0.0005

    qx_step = ( qxmax - qxmin) / 512.0
    bin_x = [ (qxmin + qx_step * float( j)) for j in range(513)] 

    qzmax = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float( xmax - xord)/2500.0))) / initial_bin
    qzmin = qzmax
    for x in xvar:
	for bin in bins:
	    QzC = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float( x - xord)/2500.0))) / bin
	    if ( qzmax < QzC):
		qzmax = QzC
	    if ( qzmin > QzC):
		qzmin = QzC
#    qzmin = 0.50
#    qzmax = 1.012

    qz_step = (qzmax - qzmin) / 512.0
    bin_z = [ (qzmin + qz_step * float( j)) for j in range(513)]

    print("Qx min  	>> ", qxmin)
    print("Qx max  	>> ", qxmax)
    # print "Qy min  	>> ", qymin
    # print "Qy max  	>> ", qymax
    print("Qz min  	>> ", qzmin)
    print("Qz max  	>> ", qzmax)

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(512)] for k in range(512)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    QxC = QC * ( math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) - math.cos(math.pi * angle/180.0))
			    QyC = QC *  math.cos(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0))
			    QzC = QC * ( math.sin(math.pi * (angle+angle2)/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) + math.sin(math.pi * angle/180.0))
			    for k in range( eventn):
				tof = float( events[k])
				if ( tof >= initial_bin and tof <= final_bin):
				    ix = 0
				    Qx = QxC / tof
				    while 1:
					if ( Qx < bin_x[ix+1]): 
					    break
					ix += 1
					if ix >= 512:
					     break
				    if ( ix < 512):
					iz = 0
					Qz = QzC / tof
					while 1:
					    if ( Qz < bin_z[iz+1]): 
						break
					    iz += 1
					    if iz >= 512:
						break
					if ( iz < 512):
					    data[ix][iz].append( QyC / tof)
	for x in range(512):
            int_list = data[x]
	    int_list_at_py = [ float( len( y)) for y in int_list ]
	    err_list_at_py = [ math.sqrt(e) for e in int_list_at_py ]
	    ec = mm.ElementContainer()
	    ec.Add( "Z", bin_z)
	    ec.Add( "Intensity", int_list_at_py )
	    ec.Add( "Error", err_list_at_py )
	    ec.SetKeys("Z","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_x[x] )
	    xr_v.append( bin_x[x + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'qx(nm^(-1))')
	p.SetYLabel( 'qz(nm^(-1))')

    return ECA

########################################
def GetHistMWPCQz(runNo=125523, initial_bin=10000.0,final_bin=38100.0,angle=-1.0, xord=-1, yord=-1, dioFlag=-1, bank=0):
    """
    Load EventData and convert to histogram for MWPC
    @param  runNo        (int) Run Number
    @param  initial_bin  (double) 
    @param  final_bin    (double) 
    @param  angle        (double) 
    @param  xord         (int)
    @param  yord         (int)
    @param  dioFlag      (int)
    @param  bank         (int)
    @retval ECA          (ElementContainerArray)
    """

    pixels = (256,256)
    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, dioFlag)

    if ( initial_bin < 1.0):
	initial_bin = 1.0
    if ( final_bin <= 1.0):
	final_bin = 38100.0
    if ( initial_bin >= final_bin):
	initila_bin = 1.0

    a2th = PickupCurpos(runNo, 'A2TH')
    if ( angle < 0.0):
	if ( a2th != -9999.9):
	    angle = float( a2th) / 2.0
	else:
	    angle = 0.3
    if ( dioFlag == -1):
	bank = 0

    print("Make ElementContainerMatrix")
    print("initial bin	>> ", initial_bin)
    print("final bin	>> ", final_bin)
    print("angle    	>> ", angle)
    print("dioFlag  	>> ", dioFlag)
    print("bank    	>> ", bank)

    # xmin,max ymin,max constant
    xmin = 32
    xmax = 224
    ymin = 32
    ymax = 224
    pix_per = 0.5

    t0c = len( T0Count)
    if ( xord == -1 and yord == -1):
	nmax = 0
	for i in range( t0c):
	    if ( i == bank):
		dataone = datalist[i]
		for y in range( len(dataone) ):
		    if ( y >= ymin and y <= ymax):
			for x in range( len(dataone[y])):
			    if ( x >= xmin and x <= xmax):
				n = len(dataone[y][x])
				n += len(dataone[y-1][x-1])
				n += len(dataone[y-1][x])
				n += len(dataone[y][x-1])
				n += len(dataone[y+1][x])
				n += len(dataone[y][x+1])
				n += len(dataone[y+1][x+1])
				n += len(dataone[y+1][x-1])
				n += len(dataone[y-1][x+1])
				if ( n > nmax):
				    nmax = n
				    xord = x
				    yord = y
	print("Auto Origin set")
    print("x origin   	>> ", xord)
    print("y origin 	>> ", yord)

    QC = 2.0 * math.pi * (18.0e7/3955.6)

    qxmax = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(xmax - xord)/2500.0)) * math.cos(math.atan(pix_per * float(ymax - yord)/2500.0)) - math.cos(math.pi * angle/180.0)) / initial_bin
    qxmin = qxmax
    xvar = [ xmin, xmax]
    yvar = [ ymin, ymax]
    bins = [initial_bin, final_bin]
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0)) / bin
		if ( qxmax < QxC):
		    qxmax = QxC
		if ( qxmin > QxC):
		    qxmin = QxC
    if ( qxmax < 0.0):
	qxmax = 0.0

    qx_step = ( qxmax - qxmin) / 256.0
    bin_x = [ (qxmin + qx_step * float( j)) for j in range(257)] 

    qymax = QC *  math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(xmin - xord)/2500.0)) * math.sin(math.atan(pix_per * float(ymin - yord)/2500.0)) / initial_bin
    qymin = qymax
    for x in xvar:
	for y in yvar:
	    for bin in bins:
		QyC = QC *  math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0)) / bin
		if ( qymax < QyC):
		    qymax = QyC
		if ( qymin > QyC):
		    qymin = QyC
    qy_step = (qymax - qymin) / 256.0
    bin_y = [ (qymin + qy_step * float( j)) for j in range(257)]

    qzmax = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * angle/180.0 + math.atan(pix_per * float( xmax - xord)/2500.0))) / initial_bin
    qzmin = qzmax
    for x in xvar:
	for bin in bins:
	    QzC = QC * (math.sin( math.pi * angle / 180.0) + math.sin( math.pi * angle/180.0 + math.atan(pix_per * float( x - xord)/2500.0))) / bin
	    if ( qzmax < QzC):
		qzmax = QzC
	    if ( qzmin > QzC):
		qzmin = QzC
    qz_step = (qzmax - qzmin) / 256.0
    bin_z = [ (qzmin + qz_step * float( j)) for j in range(257)]

    print("Qx min  	>> ", qxmin)
    print("Qx max  	>> ", qxmax)
    print("Qy min  	>> ", qymin)
    print("Qy max  	>> ", qymax)
    print("Qz min  	>> ", qzmin)
    print("Qz max  	>> ", qzmax)

    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    for i in range( t0c):
      if ( i == bank):
	t0s = T0Count[i]
	# event_c = 0
	dataone = datalist[i]
	data=[[[] for j in range(256)] for k in range(256)]
	for y in range( len(dataone) ):
	    if ( y >= ymin and y <= ymax):
		for x in range( len(dataone[y]) ):
		    if ( x >= xmin and x <= xmax):
			events = dataone[y][x]
			eventn = len( events)
			if ( eventn > 0):
			    QxC = QC * ( math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.cos(math.atan(pix_per * float(y - yord)/2500.0)) - math.cos(math.pi * angle/180.0))
			    QyC = QC *  math.cos(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) * math.sin(math.atan(pix_per * float(y - yord)/2500.0))
			    QzC = QC * ( math.sin(math.pi * angle/180.0 + math.atan(pix_per * float(x - xord)/2500.0)) + math.sin(math.pi * angle/180.0))
			    for k in range( eventn):
				tof = float( events[k])
				if ( tof >= initial_bin and tof <= final_bin):
				    ix = 0
				    Qx = QxC / tof
				    while 1:
					if ( Qx < bin_x[ix+1]): 
					    break
					ix += 1
					if ix >= 256:
					     break
				    if ( ix < 256):
					iy = 0
					Qy = QyC / tof
					while 1:
					    if ( Qy < bin_y[iy+1]): 
						break
					    iy += 1
					    if iy >= 256:
						break
					if ( iy < 256):
					    data[ix][iy].append( QzC / tof)
	for x in range(256):
            int_list = data[x]
	    int_list_at_py = [ float( len( y)) for y in int_list ]
	    err_list_at_py = [ math.sqrt(e) for e in int_list_at_py ]
	    ec = mm.ElementContainer()
	    ec.Add( "Y", bin_z)
	    ec.Add( "Intensity", int_list_at_py )
	    ec.Add( "Error", err_list_at_py )
	    ec.SetKeys("Y","Intensity","Error")
	    xr_v = mm.MakeDoubleVector()
	    xr_v.append( bin_x[x] )
	    xr_v.append( bin_x[x + 1] )
	    ec.AddToHeader( "XRANGE", xr_v )
	    ECA.Add(ec)

	p=mp.M2Plot(ECA)
	p.SetXLabel( 'qx(nm^(-1))')
	p.SetYLabel( 'qy(nm^(-1))')

    return ECA

def GetM2PlotMWPC( runNo=0, dio = -1 ):
    """
    Load EventData to produce histogram
    GetM2PlotMWPC( runNo=0, dio = -1)
    
    @param runNo     (int)     Run Number
    @param dio (int) dio mode -1, or 0, or 1
    @retval ECA (ElementContainerArray)
    """

    px_list=[ -0.5+float(i) for i in range(257) ]
    pixels = (256,256)

    datalist, T0Count = mH.getEventDataArray( runNo, "MWPC", pixels, -1)
    ECA = mm.ElementContainerArray()
    instrument = os.environ["UTSUSEMI_INST_CODE"]
    ECA.AddToHeader("INSTRUMENT", instrument )
    ECA.AddToHeader("RUNNUMBER", str(runNo))
    dataone = datalist[0]

    correct = np.loadtxt('calib_mwpc_161106.txt')

    for i_px in range(157,171):
	for i_py in range(64,76):
	    correct[i_py, i_px] = correct[i_py + 24, i_px]
    for i_px in range(230,242):
	for i_py in range(216,230):
	    correct[i_py, i_px] = correct[i_py - 24, i_px]

    #f = open("/usr/local/mlf/VNR/ana/xml/"+str(runNo).zfill(6)+".dat", 'w')
    f = open( os.path.join( getVNRConst.GetUserDir(),"ana","xml",str(runNo).zfill(6)+".dat"), 'w') #[inamura 180403]
    for i_px in range(256):
	int_list_at_py = [ float( len( dataone[i_py][i_px])) * float(correct[i_py,255 - i_px]) for i_py in range(256) ]
	err_list_at_py = [ math.sqrt(y) for y in int_list_at_py ]
    
        ec = mm.ElementContainer()
        ec.Add( "X", px_list )
        ec.Add( "Intensity", int_list_at_py )
        ec.Add( "Error", err_list_at_py )
        ec.SetKeys("X","Intensity","Error")
        xr_v = mm.MakeDoubleVector()
        xr_v.append( px_list[i_px] )
        xr_v.append( px_list[i_px+1] )
        ec.AddToHeader( "XRANGE", xr_v )
        ECA.Add(ec)
	for i_py in range(len(int_list_at_py)):
	    f.write("%d\t%d\t%f\t%f\n" % (i_px, i_py, int_list_at_py[i_py], err_list_at_py[i_py]))

    f.close()
#   import utsusemi.vis.M2Plot as mp
#   p=mp.M2Plot(dat)

    return ECA

########################################
# Dictionary for entry the name of functions
_functions = {
            "GetHistMWPCDetector":GetHistMWPCDetector,
            "GetHistRPMTDetector":GetHistRPMTDetector,
            "GetHistMWPCDetectorT":GetHistMWPCDetectorT,
            "GetHistMWPC0d":GetHistMWPC0d,
            "GetHistMWPC0dx":GetHistMWPC0dx,
            "GetHistMWPC0dTime":GetHistMWPC0dTime,
            "GetHistMWPC0dDio":GetHistMWPC0dDio,
            "GetHistMWPC0dDiox":GetHistMWPC0dDiox,
            "GetHistMWPCTOF":GetHistMWPCTOF,
            "GetHistMWPCX":GetHistMWPCX,
            "GetHistMWPCX2":GetHistMWPCX2,
            "GetHistMWPCQy":GetHistMWPCQy,
            "GetHistMWPCQy2":GetHistMWPCQy2,
            "GetHistMWPCQyoff":GetHistMWPCQyoff,
            "GetHistMWPCQz":GetHistMWPCQz,
            "GetM2PlotMWPC":GetM2PlotMWPC
            }
_functionsOrder = [
    "GetHistMWPCDetector",
    "GetHistRPMTDetector",
    "GetHistMWPCDetectorT",
    "GetHistMWPC0d",
    "GetHistMWPC0dx",
    "GetHistMWPC0dTime",
    "GetHistMWPC0dDio",
    "GetHistMWPC0dDiox",
    "GetHistMWPCTOF",
    "GetHistMWPCX",
    "GetHistMWPCX2",
    "GetHistMWPCQy",
    "GetHistMWPCQy2",
    "GetHistMWPCQyoff",
    "GetHistMWPCQz",
    "GetM2PlotMWPC"
    ]
    



#! /usr/bin/env python3
from __future__ import print_function
import os
import sys
import DR
import Cmm
import utsusemi.ana.Reduction.BaseCommandsInEla as BC

#####################
# modify here
####################
Ei = 30.04
delta_hw = 0.2
hw_min = -5.0
hw_max = 28.0
maskfile = "mask.txt"
normFactor = 1000000.0
XtalParam = "XtalParam.xml"
savePathDir = "./data"
gonioValueAtZero = 31.032
#########################

argv = sys.argv
print("len(argv)={}".format(len(argv)))

listFileName = ""
if len(argv) == 2:
    if os.path.exists(argv[1]):
        listFileName = argv[1]
        print("AngleList={}".format(listFileName))
    else:
        raise UserWarning("Not found file=%s" % (argv[1]))
else:
    print("Usage::")
    print("        $ ./%s <filename of runlist>" % (argv[0]))
    print("        ")
    print("        $ python %s <filename of runlist>" % (argv[0]))
    sys.exit(0)

if not os.path.exists(savePathDir):
    print(" savePathDir = {} is not found. Making new ".format(savePathDir))
    try:
        os.makedirs(savePathDir)
    except:
        raise UserWarning("Cannot make new folder(" + savePathDir + ")")

paramDic = {"normFactor": normFactor, "gonio": gonioValueAtZero}
AngleList, condsDict = BC.LoadRunListToExtract(listFileName, paramDic, mergeAngles=True)

for angle_key in AngleList:
    angle = float(angle_key)
    runNos = condsDict[angle_key][0]
    norm = condsDict[angle_key][1]
    print("==============[ %s:%8.3f,%10.5f ]" % (runNos, angle, norm))
    DAT = DR.GetDataOfMonochroEi3(runNos, Ei, delta_hw, "%g %g" % (hw_min, hw_max), MaskFile=maskfile, NormFactor=norm)
    savePath = os.path.join(savePathDir, ("Ang_%s.vbin" % (angle_key)))  # save file name "Ang_123.450.vbin"
    Cmm.VisContMOutputFileByPhi(DAT, XtalParam, angle, savePath)
    del DAT



# -*- coding: utf-8 -*-
from __future__ import print_function
import sys
#from pyqtgraph.Qt import QtCore, QtGui, uic
#from PySide import QtCore
try:
    import PySide6
    from PySide6 import QtCore, QtGui, QtWidgets
except:
    import PySide2
    from PySide2 import QtCore, QtGui, QtWidgets
#from pyqtgraph.Qt import QtCore,QtGui,QtWidgets
import pyqtgraph as pg
import pyqtgraph.opengl as gl
import numpy as np
from OpenGL.GL import *
import SliceData
import FreeSliceData
import ViewSubData

import Manyo
#import Manyo.Utsusemi
import PlaneSlicer
import uGao.M2PlotPlusCui as mp
import threading
import math
import time
import os
import subprocess

class ViewMainData(QtCore.QObject):
    SLICE_XY = 0
    SLICE_YZ = 1
    SLICE_ZX = 2

    PROCESS_MAIN       = 0
    PROCESS_M2PLOT     = 1
    PROCESS_OUT_IMAGE  = 2
    PROCESS_OUT_BINARY = 3

    DATA_ORI_XYZ = 0
    DATA_ORI_XZY = 1
    DATA_ORI_YXZ = 2
    DATA_ORI_YZX = 3
    DATA_ORI_ZXY = 4
    DATA_ORI_ZYX = 5

    HEADKEY_SLICE3DKEYS = "SLICE3DKEYS"
    HEADKEY_SLICE3DBINS = "SLICE3DBINS"

    update_scrollbar_signal   = QtCore.Signal()
    update_scrollbar_signal   = QtCore.Signal()
    update_scrollbar_range_signal = QtCore.Signal()
    update_data_info_signal   = QtCore.Signal()
    update_basic_slice_signal = QtCore.Signal()
    update_axis_grid_signal   = QtCore.Signal()
    status_bar_signal         = QtCore.Signal(str)
    create_slice_signal       = QtCore.Signal(tuple)
    create_free_slice_signal  = QtCore.Signal(tuple)
    update_slice_view_signal  = QtCore.Signal()
    update_glview_signal      = QtCore.Signal()
    save_glview_signal        = QtCore.Signal(str)
    update_autoscale_signal   = QtCore.Signal()

    update_grid_size_signal   = QtCore.Signal()
    update_grid_signal        = QtCore.Signal()
    def __init__(self,ecm_in=None):
        super(ViewMainData, self).__init__()
        self.filePath = ""
        self.stop_event = threading.Event()
        self.data_xy = np.zeros((1, 1))
        self.data_xy_pos = 0.0
        self.data_xy_range = [0.0, 1.0, 0.0, 1.0]
        self.data_yz = np.zeros((1, 1))
        self.data_yz_pos = 0.0
        self.data_yz_range = [0.0, 1.0, 0.0, 1.0]
        self.data_zx = np.zeros((1, 1))
        self.data_zx_pos = 0.0
        self.data_zx_range = [0.0, 1.0, 0.0, 1.0]
        self.data_ax1bin = 1.0
        self.data_ax2bin = 1.0
        self.data_ax3bin = 1.0
        self.data_free = np.zeros((1, 1))

        self.intensityMin = 9999.0
        self.intensityMax = -9999.0
        self.imageID = 0

        self.basicSliceAx1From = 0.0
        self.basicSliceAx1To   = 0.0
        self.basicSliceAx2From = 0.0
        self.basicSliceAx2To   = 0.0
        self.basicSliceAx3From = 0.0
        self.basicSliceAx3To   = 0.0

        # 3面スライス用共通データ
        self.basicSliceAx1bin = 0.01
        self.basicSliceAx2bin = 0.01
        self.basicSliceAx3bin = 0.01

        self.basicSliceAx1Scale = 1.0
        self.basicSliceAx2Scale = 1.0
        self.basicSliceAx3Scale = 1.0

        # 3面スライス用データ(各軸)
        self.ax1Slice = SliceData.SliceData()
        self.ax1Slice.axisTitle = "ax1"
        self.ax2Slice = SliceData.SliceData()
        self.ax2Slice.axisTitle = "ax2"
        self.ax3Slice = SliceData.SliceData()
        self.ax3Slice.axisTitle = "ax3"

        self.slicePlane = ViewMainData.SLICE_XY


        self.freeSlice = FreeSliceData.FreeSliceData()

        self.ax1Key = "Xval"
        self.ax2Key = "Yval"
        self.ax3Key = "Zval"

        #---------
        self.update_slice_view_flag = False
        # signals

        # [tito 20220519] comment out
        """
        self.update_scrollbar_signal   = QtCore.SIGNAL("update_scrollbar_signal")
        self.update_scrollbar_range_signal = QtCore.SIGNAL("update_scrollbar_range_signal")
        self.update_data_info_signal   = QtCore.SIGNAL("update_data_information_signal")
        self.update_basic_slice_signal = QtCore.SIGNAL("update_basic_slice_signal")
        self.update_axis_grid_signal   = QtCore.SIGNAL("update_axis_grid_signal")
        self.status_bar_signal         = QtCore.SIGNAL("status_bar_signal")
        self.create_slice_signal       = QtCore.SIGNAL("create_slice_signal")
        self.create_free_slice_signal  = QtCore.SIGNAL("create_free_slice_signal")
        self.update_slice_view_signal  = QtCore.SIGNAL("update_slice_view_signal")
        self.update_glview_signal      = QtCore.SIGNAL("update_glview_signal")
        self.save_glview_signal        = QtCore.SIGNAL("save_glview_signal")
        self.update_autoscale_signal   = QtCore.SIGNAL("update_autoscale_signal")

        self.update_grid_size_signal   = QtCore.SIGNAL("update_grid_size_signal")
        self.update_grid_signal        = QtCore.SIGNAL("update_grid_signal")
        """

        #[inamura 170227]-->
        self.data_ecm = None
        if (ecm_in!=None):
            self.SetData( ecm_in )
        #<--[inamura 170227]

    #[inamura 170227]-->
    #[inamura 180422]
    def SetData(self, ecm_in, xkey="-", ykey="-", zkey="-", isFromFile=False, xbin="-", ybin="-", zbin="-"):
        if self.data_ecm!=None:
            del self.data_ecm

        self.data_ecm = ecm_in

        if not isFromFile:
            self.filePath="Direct"

        ecm_hh = self.data_ecm.PutHeaderPointer()
        if ( xkey=="-" and ykey=="-" and zkey=="-" ):
            if ecm_hh.CheckKey(ViewMainData.HEADKEY_SLICE3DKEYS)==1:
                key_vect = ecm_hh.PutStringVector(ViewMainData.HEADKEY_SLICE3DKEYS)
                self.ax1Key = key_vect[0]
                self.ax2Key = key_vect[1]
                self.ax3Key = key_vect[2]
            else:
                self.data_ecm = None
                return
        else:
            self.ax1Key = xkey
            self.ax2Key = ykey
            self.ax3Key = zkey


        #self.rangex = [-0.5, 3.0]
        #self.rangey = [-1.0, 1.5]
        #self.rangez = [0.1, 30.1]

        #self.getDataInformation()
        self.getDataInformation2(xbin,ybin,zbin) #[inamura 180422]
        self.getDataKeys()

        # set view scale
        lenX = float(self.rangex[1] - self.rangex[0])
        lenY = float(self.rangey[1] - self.rangey[0])
        lenZ = float(self.rangez[1] - self.rangez[0])

        scaleExp = 3
        if lenX < lenY and lenX < lenZ:
           self.basicSliceAx1Scale = 1.0
           self.basicSliceAx2Scale = lenX/lenY
           self.basicSliceAx3Scale = lenX/lenZ
        elif lenY < lenZ:
           self.basicSliceAx1Scale = lenY/lenX
           self.basicSliceAx2Scale = 1.0
           self.basicSliceAx3Scale = lenY/lenZ
        else:
           self.basicSliceAx1Scale = lenZ/lenX
           self.basicSliceAx2Scale = lenZ/lenY
           self.basicSliceAx3Scale = 1.0

        self.basicSliceAx1Scale = round(self.basicSliceAx1Scale, scaleExp)
        self.basicSliceAx2Scale = round(self.basicSliceAx2Scale, scaleExp)
        self.basicSliceAx3Scale = round(self.basicSliceAx3Scale, scaleExp)

        self.basicSliceAx1From = self.rangex[0]
        self.basicSliceAx1To   = self.rangex[1]
        self.basicSliceAx2From = self.rangey[0]
        self.basicSliceAx2To   = self.rangey[1]
        self.basicSliceAx3From = self.rangez[0]
        self.basicSliceAx3To   = self.rangez[1]

        self.ax1Slice.thickness = self.basicSliceAx1bin * 1.0
        self.ax2Slice.thickness = self.basicSliceAx2bin * 1.0
        self.ax3Slice.thickness = self.basicSliceAx3bin * 1.0


        lenX = self.rangex[1] - self.rangex[0]
        lenY = self.rangey[1] - self.rangey[0]
        lenZ = self.rangez[1] - self.rangez[0]

        self.UpdatePlotter() #[inamura 180422]

    #[inamura 180422]-->
    def UpdatePlotter(self):
        self.update_data_info_signal.emit()
        self.update_basic_slice_signal.emit()
        self.update_axis_grid_signal.emit()
        self.update_scrollbar_range_signal.emit()
        QtWidgets.QApplication.restoreOverrideCursor()
    #<--[inamura 180422]

    def ReadSerializationFileBinary(self, filePath):
        QtWidgets.QApplication.setOverrideCursor(QtGui.QCursor(QtCore.Qt.WaitCursor))
        self.filePath = filePath
        data_ecm = None
        root, ext = os.path.splitext(str(filePath))
        if ext == ".txt":
            #data_ecm = Manyo.ElementContainerMatrix()
            #tt = Manyo.Utsusemi.UtsusemiContainerTextConverter()
            #tt.Load( str(filePath), data_ecm )
            #del tt
            (ECM, bins, mins, maxs) = self.ReadFromTextFile(str(filePath))
            self.SetData(ecm_in=ECM, xkey="-", ykey="-", zkey="-", isFromFile=True, xbin=str(bins[0]), ybin=str(bins[1]), zbin=str(bins[2]))
            return
        elif ext == ".srlz":
            data_ecm = Manyo.ElementContainerMatrix()
            file = Manyo.ReadSerializationFileBinary(str(filePath))
            file.Load(data_ecm)
            del file
        elif ext == ".mdb":
            ff = Manyo.ManyoDataIO()
            data_ecm = ff.ReadElementContainerMatrix(str(filePath))
            del ff

        if data_ecm.PutHeaderPointer().CheckKey(ViewMainData.HEADKEY_SLICE3DBINS) == 1:
            bins = data_ecm.PutHeaderPointer().PutDoubleVector(ViewMainData.HEADKEY_SLICE3DBINS)
            self.SetData(ecm_in=data_ecm, xkey="-", ykey="-", zkey="-", isFromFile=True, xbin=str(bins[0]), ybin=str(bins[1]), zbin=str(bins[2]))
            return
        self.SetData(data_ecm)
    #<--[inamura 170227]

    def ReadFromTextFile(self, filepath):
        bins = [0.0, 0.0, 0.0]
        mins = [1e10, 1e10, 1e10, 1e10]
        maxs = [-1e10, -1e10, -1e10, -1e10]
        fo = open(filepath, "r")
        vx = []
        vy = []
        vz = []
        ii = []
        ee = []
        if fo is None:
            print("Failed open given file : {}".format(filepath))
        # Read Text file
        while(True):
            aline = fo.readline()
            if aline == "":
                break
            # If comment out line or header info
            if aline.find("# ") == 0:
                if aline.find("BINS") != 0:
                    v0 = aline.split("=")
                    if len(v0) == 2:
                        v = v0[1].split(",")
                        if len(v) == 3:
                            bins[0] = float(v[0])
                            bins[1] = float(v[1])
                            bins[2] = float(v[2])
                        else:
                            print("Header BINS format is not valid (BINS = <x_bin>, <y_bin>, <z_bin>)")
            else:
                aline = aline.strip()
                v = aline.split(",")
                if len(v) != 5:
                    print("Invalid format = {}".format(aline))
                    continue
                vv = []
                try:
                    for av in v:
                        vv.append(float(av))
                except:
                    print("Infalid format for float conversions")
                    continue

                for i in range(4):
                    if vv[i] < mins[i]:
                        mins[i] = vv[i]
                    if vv[i] > maxs[i]:
                        maxs[i] = mins[i]

                vx.append(vv[0])
                vy.append(vv[1])
                vz.append(vv[2])
                ii.append(vv[3])
                ee.append(vv[4])
        fo.close()
        EC = Manyo.ElementContainer()
        vx_key = "Vx"
        vy_key = "Vy"
        vz_key = "Vz"
        ii_key = "Intensity"
        ee_key = "Error"
        EC.Add(vx_key, vx)
        EC.Add(vy_key, vy)
        EC.Add(vz_key, vz)
        EC.Add(ii_key, ii)
        EC.Add(ee_key, ee)
        EC.SetKeys(vz_key, ii_key, ee_key)
        ECA = Manyo.ElementContainerArray()
        ECA.Add(EC)
        hh = Manyo.HeaderBase()
        vst = Manyo.MakeStringVector()
        vst.push_back(vx_key)
        vst.push_back(vy_key)
        vst.push_back(vz_key)
        hh.Add(ViewMainData.HEADKEY_SLICE3DKEYS, vst)
        ECM = Manyo.ElementContainerMatrix(hh)
        ECM.Add(ECA)
        return (ECM, bins, mins, maxs)

    def getDataInformation(self):
        if not hasattr(self, 'data_ecm'):
            return

        ecaNum = self.data_ecm.PutSize()
        eca0 = self.data_ecm.Put(0)
        ecNum = eca0.PutSize()

        ec0  = eca0.Put(0)

        xmin = ec0.Min(str(self.ax1Key))
        xmax = ec0.Max(str(self.ax1Key))
        ymin = ec0.Min(str(self.ax2Key))
        ymax = ec0.Max(str(self.ax2Key))
        zmin = ec0.Min(str(self.ax3Key))
        zmax = ec0.Max(str(self.ax3Key))

        ec1 = eca0.Put(1)

        if xmin != xmax:
            xlist = ec0.PutList(str(self.ax1Key))
            xmin1 = ec1.Min(str(self.ax2Key))
            if xmin != xmin1:
               data_orientation = ViewMainData.DATA_ORI_XYZ
               ylist = [0] * ecNum
               zlist = [0] * ecaNum
            else:
               data_orientation = ViewMainData.DATA_ORI_XZY
               ylist = [0] * ecaNum
               zlist = [0] * ecNum
        elif ymin != ymax:
            ylist = ec0.PutList(str(self.ax2Key))
            xmin1 = ec1.Min(str(self.ax1Key))
            if xmin != xmin1:
               data_orientation = ViewMainData.DATA_ORI_YXZ
               xlist = [0] * ecNum
               zlist = [0] * ecaNum
            else:
               data_orientation = ViewMainData.DATA_ORI_YZX
               xlist = [0] * ecaNum
               zlist = [0] * ecNum
        elif zmin != zmax:
            zlist = ec0.PutList(str(self.ax3Key))
            xmin1 = ec1.Min(str(self.ax1Key))
            if xmin != xmin1:
               data_orientation = ViewMainData.DATA_ORI_ZXY
               xlist = [0] * ecNum
               ylist = [0] * ecaNum
            else:
               data_orientation = ViewMainData.DATA_ORI_ZYX
               xlist = [0] * ecaNum
               ylist = [0] * ecNum

        for id in range(ecNum):
            if data_orientation == ViewMainData.DATA_ORI_ZXY or \
               data_orientation == ViewMainData.DATA_ORI_YXZ:
                xlist[id] = eca0(id).Min(str(self.ax1Key))
            elif data_orientation == ViewMainData.DATA_ORI_XYZ or \
                 data_orientation == ViewMainData.DATA_ORI_ZYX:
                ylist[id] = eca0(id).Min(str(self.ax2Key))
            elif data_orientation == ViewMainData.DATA_ORI_XZY or \
                 data_orientation == ViewMainData.DATA_ORI_YZX:
                zlist[id] = eca0(id).Min(str(self.ax3Key))

        for id in range(ecaNum):
            if data_orientation == ViewMainData.DATA_ORI_ZYX or \
               data_orientation == ViewMainData.DATA_ORI_YZX:
                xlist[id] = self.data_ecm(id, 0).Min(str(self.ax1Key))
            elif data_orientation == ViewMainData.DATA_ORI_XZY or \
                 data_orientation == ViewMainData.DATA_ORI_ZXY:
                ylist[id] = self.data_ecm(id, 0).Min(str(self.ax2Key))
            elif data_orientation == ViewMainData.DATA_ORI_XYZ or \
                 data_orientation == ViewMainData.DATA_ORI_YXZ:
                zlist[id] = self.data_ecm(id, 0).Min(str(self.ax3Key))



        # get minimum bin size
        size = len(xlist)
        xlist.sort() #[inamura 180422]
        tmp1 = np.array(xlist[0:size-1])
        tmp2 = np.array(xlist[1:size])
        tmp3 = tmp2 - tmp1
        self.basicSliceAx1bin = min(tmp3)

        size = len(ylist)
        ylist.sort() #[inamura 180422]
        tmp1 = np.array(ylist[0:size-1])
        tmp2 = np.array(ylist[1:size])
        tmp3 = tmp2 - tmp1
        self.basicSliceAx2bin = min(tmp3)

        size = len(zlist)
        zlist.sort() #[inamura 180422]
        tmp1 = np.array(zlist[0:size-1])
        tmp2 = np.array(zlist[1:size])
        tmp3 = tmp2 - tmp1
        self.basicSliceAx3bin = min(tmp3)
        del tmp1, tmp2, tmp3

        # get xrange, yrange, zrange
        self.rangex = [min(xlist), max(xlist)]
        self.rangey = [min(ylist), max(ylist)]
        self.rangez = [min(zlist), max(zlist)]
        #print "len(xlist), len(ylist), len(zlist)=",len(xlist), len(ylist), len(zlist)
        del xlist, ylist, zlist

    def getDataInformationIntensity(self):
        if not hasattr(self, 'data_ecm'):
            return
        eca   = self.data_ecm.Put(0)
        ecaNum = self.data_ecm.PutSize()
        ecNum = eca.PutSize()
        minIntensity = np.zeros((ecaNum, ecNum))
        maxIntensity = np.zeros((ecaNum, ecNum))
        for id in range(ecaNum):
            eca = self.data_ecm(id)
            if eca.PutHeaderPointer().CheckKey("MASKED") == 1:
                if eca.PutHeaderPointer().PutInt4("MASKED") == 1:
                    continue
            ecNum = eca.PutSize()
            for i in range(ecNum):
                """
                ec = eca(i)
                intensity = ec.PutList("Intensity")
                #intensity = ec.PutYList()
                error     = ec.PutList("Error")
                tmp  = np.array(intensity)
                maxIntensity[id, i] = tmp.max()
                tmp2 = tmp[np.where(tmp < 1.0)]
                if tmp2.size > 0:
                    minIntensity[id, i] = tmp2.min()
                    #maxIntensity[id, i] = tmp.max()
                """
                if eca(i).PutHeaderPointer().CheckKey("MASKED") == 1:
                    if eca(i).PutHeaderPointer().PutInt4("MASKED") == 1:
                        continue
                minIntensity[id, i] = eca(i).Min("Intensity")
                maxIntensity[id, i] = eca(i).Max("Intensity")

        self.intensityMin = minIntensity.min()
        self.viewSubData.scalarRangeMin = self.intensityMin
        self.intensityMax = maxIntensity.max()
        self.viewSubData.scalarRangeMax = self.intensityMax

    #[inamura 180422]-->
    def getDataInformation2(self,in_xbin,in_ybin,in_zbin):
        if in_xbin=="-":
            self.getDataInformation()
        else:
            xbin = -1.0
            ybin = -1.0
            zbin = -1.0
            if in_xbin!="-":
                xbin = float(in_xbin)
            if in_ybin!="-":
                ybin = float(in_ybin)
            if in_zbin!="-":
                zbin = float(in_zbin)
            if (xbin>0.) and (ybin<0.0):
                ybin = xbin
            if (xbin>0.) and (zbin<0.0):
                zbin = xbin

            self.basicSliceAx1bin = xbin
            self.basicSliceAx2bin = ybin
            self.basicSliceAx3bin = zbin

        try:
            import Manyo.MLF as mm
            p = mm.MlfMatrixSlicer(self.data_ecm)
            p.SetAxes(str(self.ax1Key), str(self.ax2Key), str(self.ax3Key) )
            v = p.PutAllAxesRanges()
            del p
            xMin = v[0]
            xMax = v[1]
            yMin = v[2]
            yMax = v[3]
            zMin = v[4]
            zMax = v[5]
            if in_xbin!="-":
                xMin = math.floor(xMin/xbin)*xbin
                xMax = math.floor(xMax/xbin)*xbin
                yMin = math.floor(yMin/ybin)*ybin
                yMax = math.floor(yMax/ybin)*ybin
                zMin = math.floor(zMin/zbin)*zbin
                zMax = math.floor(zMax/zbin)*zbin
            self.rangex = [ xMin, xMax ]
            self.rangey = [ yMin, yMax ]
            self.rangez = [ zMin, zMax ]
        except:
            pass
        #<--[inamura 180422]

        self. getDataInformationIntensity()

    def getDataKeys(self):
        #self.ax1Key = "Xval"
        #self.ax2Key = "Yval"
        #self.ax3Key = "Zval"

        self.ax1Slice.axisTitle = self.ax1Key
        self.ax2Slice.axisTitle = self.ax2Key
        self.ax3Slice.axisTitle = self.ax3Key

    def isThreadRunning(self):
        running = False
        if hasattr(self, "sliceThread"):
           if self.sliceThread.is_alive():
               running = True

        return running

        #tlist = threading.enumerate()
        #if 1 < len(tlist):
        #     return True
        #else:
        #     return False
        #for t in tlist:
        #        print t.name

    def getThreadName(self, process, planeID):
        procName = ""
        if process == ViewMainData.PROCESS_MAIN:
           procName = "main"
        elif process == ViewMainData.PROCESS_M2PLOT:
           procName = "m2plot"
        elif process == ViewMainData.PROCESS_OUT_IMAGE:
           procName = "image"
        elif process == ViewMainData.PROCESS_OUT_BINARY:
           procName = "binary"

        procName += "-%d" % planeID
        return procName

    def doFreeSliceView(self, process, outputPath):
        if self.isThreadRunning():
            #QtGui.QMessageBox.warning(self, 'Error', 'Other thread is running. Please wait.')
            return False

        if process == ViewMainData.PROCESS_M2PLOT:
            self.showFreeSliceView(process, outputPath)
        else:
            self.sliceThread = threading.Thread(target = self.showFreeSliceView, \
                                                args=(process, outputPath))
            self.sliceThread.start()
        return True

    def showFreeSliceView(self, process, outputPath):
        QtWidgets.QApplication.setOverrideCursor(QtGui.QCursor(QtCore.Qt.WaitCursor))
        self.stop_event.clear()
        ps = PlaneSlicer.PlaneSlicer(self.data_ecm)

        org = self.freeSlice.org
        x   = self.freeSlice.x
        y   = self.freeSlice.y
        xbin   = self.freeSlice.xbin
        ybin   = self.freeSlice.ybin
        rngx  = self.freeSlice.rangex
        rngy  = self.freeSlice.rangey
        thick = self.freeSlice.thickness

        ps.SetAxes(str(self.ax1Key), str(self.ax2Key), str(self.ax3Key) ) #[inamura 180422]
        ps.SetPlaneKeys(str(self.ax1Key), str(self.ax2Key) ) #[inamura 180422]

        eca  = ps.GetPlane(org, x, y, xbin, ybin, rngx, rngy, thick)

        root, ext = os.path.splitext(str(outputPath))

        current = 0
        if process == ViewMainData.PROCESS_M2PLOT:
            p    = mp.M2PlotPlusCui(eca)
#            subprocess.call("mp.M2PLot(eca)")
        elif process == ViewMainData.PROCESS_OUT_BINARY:
            path = root + "-f%03d.eca" % (current -1)
            eca.SaveToBinFile("Intensity", path)
            #filePath = "‾/test.srlz"
            #write_binary = Manyo.WriteSerializationFileBinary(str(filePath))
            #write_binary.Save(eca)
        elif process == ViewMainData.PROCESS_MAIN:
            self.extractIntensityForFreeSlice(eca)
            self.create_free_slice_signal.emit((0,))
            self.update_glview_signal.emit()
#             if self.update_slice_view_flag:
#                 self.emit(self.update_slice_view_signal, (planeID, ))
#             time.sleep(1)
        elif process == ViewMainData.PROCESS_OUT_IMAGE:
            self.extractIntensityForFreeSlice(eca)
            self.create_free_slice_signal.emit((0,))
            self.update_glview_signal.emit()

#             if self.update_slice_view_flag:
#                 self.emit(self.update_slice_view_signal, (planeID, ))
# #            path = root + "%03d.png" % (current -1)
#             path = root + "%03d.png" % self.imageID
#             self.emit(self.save_glview_signal, path)
#             time.sleep(1)

#        message = "%d/%d done  @ %s = %g" % (current, totalNum, posStr, pos)
#        self.emit(self.status_bar_signal, message)

#        self.emit(self.update_scrollbar_signal)

        QtWidgets.QApplication.restoreOverrideCursor()


    def doBasicSliceView(self, process, planeID, outputPath, ax1Changed = False, ax2Changed = False, ax3Changed = False):
        if self.isThreadRunning():
            #QtGui.QMessageBox.warning(self, 'Error', 'Other thread is running. Please wait.')
            return False

        #threadName = self.getThreadName(process, planeID)
        #self.sliceThread = threading.Thread(target = self.showBasicSliceViewThread,
        #                   args=(process, planeID, outputPath), name=threadName)
        #self.sliceThread.start()
        if process == ViewMainData.PROCESS_M2PLOT:
            self.showBasicSliceView(process, planeID, outputPath)
        else:
            threadName = self.getThreadName(process, planeID)
            self.sliceThread = threading.Thread(target = self.showBasicSliceViewThread, \
                                                args=(process, planeID, outputPath, ax1Changed, ax2Changed, ax3Changed), name=threadName)
            self.sliceThread.start()
        return True

    def showBasicSliceViewThread(self, process, planeID, outputPath, ax1Changed, ax2Changed, ax3Changed):
        self.imageID = 0
#        if process == ViewMainData.PROCESS_MAIN:
        if process == ViewMainData.PROCESS_MAIN or \
           process == ViewMainData.PROCESS_OUT_IMAGE:
            if ax3Changed:
                planeID = ViewMainData.SLICE_XY
                self.showBasicSliceView(process, planeID, outputPath)
            if ax1Changed:
                planeID = ViewMainData.SLICE_YZ
                self.showBasicSliceView(process, planeID, outputPath)
            if ax2Changed:
                planeID = ViewMainData.SLICE_ZX
                self.showBasicSliceView(process, planeID, outputPath)
        else:
            self.showBasicSliceView(process, planeID, outputPath)


    def showBasicSliceView(self, process, planeID, outputPath):
        QtWidgets.QApplication.setOverrideCursor(QtGui.QCursor(QtCore.Qt.WaitCursor))
        self.stop_event.clear()
        ps = PlaneSlicer.PlaneSlicer(self.data_ecm)

        pos = 0.0
        org = []
        x   = []
        y   = []
        xbin = 0.01
        ybin = 0.01
        rngx = []
        rngy = []
        thick = 1.0

        pos_str = ""
        pos_min  = 0.0
        pos_max  = 0.0
        pos_step = 0.0

        self.data_ax1bin = self.basicSliceAx1bin
        self.data_ax2bin = self.basicSliceAx2bin
        self.data_ax3bin = self.basicSliceAx3bin

        ps.SetAxes(str(self.ax1Key), str(self.ax2Key), str(self.ax3Key) ) #[inamura 180422]
        if planeID == ViewMainData.SLICE_XY:
            #ps.SetAxes(self.ax1Key, self.ax2Key, self.ax3Key)
            ps.SetPlaneKeys(str(self.ax1Key), str(self.ax2Key)) #[inamura 180422]
            pos_str = self.ax3Slice.position
            xbin = self.basicSliceAx1bin
            ybin = self.basicSliceAx2bin
            rngx = self.rangex
            rngy = self.rangey
            #rngx = [self.basicSliceAx1From, self.basicSliceAx1To]
            #rngy = [self.basicSliceAx2From, self.basicSliceAx2To]
            thick = self.ax3Slice.thickness
        elif planeID == ViewMainData.SLICE_YZ:
            #ps.SetAxes(self.ax2Key, self.ax3Key, self.ax1Key)
            ps.SetPlaneKeys(str(self.ax2Key), str(self.ax3Key)) #[inamura 180422]
            pos_str = self.ax1Slice.position
            xbin = self.basicSliceAx2bin
            ybin = self.basicSliceAx3bin
            rngx = self.rangey
            rngy = self.rangez
            thick = self.ax1Slice.thickness
        elif planeID == ViewMainData.SLICE_ZX:
            #ps.SetAxes(self.ax3Key, self.ax1Key, self.ax2Key)
            ps.SetPlaneKeys(str(self.ax1Key), str(self.ax3Key)) #[inamura 180422]
            pos_str = self.ax2Slice.position
            xbin = self.basicSliceAx1bin
            ybin = self.basicSliceAx3bin
            rngx = self.rangex
            rngy = self.rangez
            thick = self.ax2Slice.thickness

        pos_list = pos_str.split(":")
        if len(pos_list) == 3:
            pos_min = float(pos_list[0])
            pos_max = float(pos_list[1])
            pos_step = abs(float(pos_list[2]))
        else:
            pos_min = pos_max = float(pos_list[0])
            pos_step = 1.0


        pos_list = []
        if pos_min<pos_max:
            tmppos = pos_min
            while tmppos <= pos_max:
                pos_list.append( tmppos )
                tmppos += pos_step
        elif pos_min>pos_max:
            tmppos = pos_min
            while tmppos >= pos_max:
                pos_list.append( tmppos )
                tmppos -= pos_step
        elif pos_min==pos_max:
            pos_list.append( pos_min )

        totalNum = len(pos_list)

        root, ext = os.path.splitext(str(outputPath))
        current = 1
        posStr = ""
        x0 = rngx[0]-xbin/2.0 #[inamura 170227]
        y0 = rngy[0]-ybin/2.0 #[inamura 170227]
        for pos in pos_list:
            if self.stop_event.is_set():
                self.stop_event.clear()
                break

            if planeID == ViewMainData.SLICE_XY:
#                org = [0.0, 0.0, pos]
#                x   = [1.0, 0.0, pos]
#                y   = [0.0, 1.0, pos]

                org = [x0,     y0, pos]
                x   = [x0+1.0, y0, pos]
                y   = [x0, y0+1.0, pos]
                posStr = "z"
            elif planeID == ViewMainData.SLICE_YZ:
#                org = [pos, 0.0, 0.0]
#                x   = [pos, 1.0, 0.0]
#                y   = [pos, 0.0, 1.0]
                org = [pos, x0,     y0]
                x   = [pos, x0+1.0, y0]
                y   = [pos, x0, y0+1.0]
                posStr = "x"
            elif planeID == ViewMainData.SLICE_ZX:
#                org = [0.0, pos, 0.0]
#                x   = [1.0, pos, 0.0]
#                y   = [0.0, pos, 1.0]
                org = [x0,     pos, y0]
                x   = [x0+1.0, pos, y0]
                y   = [x0,     pos, y0+1.0]
                posStr = "y"

            eca  = ps.GetPlane(org, x, y, xbin, ybin, rngx, rngy, thick)
            #print "org = ", org
            #print "x = ", x
            #print "y = ", y
            #print "xbin, ybin = ", xbin, ybin
            #print "rngx = ", rngx
            #print "rngy = ", rngy
            #print "thick = ", thick

            if process == ViewMainData.PROCESS_M2PLOT:
                p    = mp.M2PlotPlusCui(eca)
#                subprocess.call("mp.M2PLot(eca)")
            elif process == ViewMainData.PROCESS_OUT_BINARY:
                path = root + "%03d.eca" % (current -1)
                eca.SaveToBinFile("Intensity", path)
                #filePath = "‾/test.srlz"
                #write_binary = Manyo.WriteSerializationFileBinary(str(filePath))
                #write_binary.Save(eca)
            elif process == ViewMainData.PROCESS_MAIN:
                self.extractIntensity(eca, pos, planeID)
#                self.emit(self.create_slice_signal, (planeID, pos))
                self.create_slice_signal.emit((planeID,))
                self.update_grid_size_signal.emit()
                self.update_grid_signal.emit()
                self.update_glview_signal.emit()
                if self.update_slice_view_flag:
                    self.update_slice_view_signal.emit((planeID, ))
                time.sleep(1)
            elif process == ViewMainData.PROCESS_OUT_IMAGE:
                self.extractIntensity(eca, pos, planeID)
                self.create_slice_signal.emit((planeID,))
                self.update_glview_signal.emit()
                if self.update_slice_view_flag:
                    self.update_slice_view_signal.emit((planeID, ))
#                path = root + "%03d.png" % (current -1)
                path = root + "%03d.png" % self.imageID
                self.save_glview_signal.emit(path)
                time.sleep(1)

            message = "%d/%d done  @ %s = %g" % (current, totalNum, posStr, pos)
            self.status_bar_signal.emit(message)
            current += 1
            self.imageID += 1

        self.update_scrollbar_signal.emit()
        QtWidgets.QApplication.restoreOverrideCursor()

    def extractIntensityForFreeSlice(self, eca):
        ecNum = eca.PutSize()
        x = np.array([0])
        #print ecNum

        ec = eca.Put(0)
        y  = ec.PutXList()
        ylen = len(y)

        xval = eca.PutHeader().PutDoubleVector(self.ax1Key)
        xval_list = Manyo.DoubleVectorToList(xval)

        x = xval_list
        xlen = len(x)

        # print len(x)
        # print x
        # print len(y)
        # print y

        self.data_free = np.zeros((xlen, ylen))

        tmpMin = 9999.0
        tmpMax = -9999.0
        #self.intensityMin = tmpMin
        #self.intensityMax = tmpMax

        for xid in range(xlen):
           ec = eca.Put(xid)
           intensity = ec.PutYList()
           #tmpMin = min(intensity)
           #tmpMax = max(intensity)

           for yid in range(len(intensity)):
              val = intensity[yid]
              if val > 1.0: # 1e+100 ... Invalid Value ???
                  val = 1e-16

              if val < tmpMin:
                  tmpMin = val
              elif tmpMax < val:
                  if val <= 1.0:
                      tmpMax = val
              self.data_free[xid,yid] = val

#           if tmpMin < self.intensityMin:
#               self.intensityMin = tmpMin
#           if self.intensityMax < tmpMax:
#               self.intensityMax = tmpMax





    def extractIntensity(self, eca, pos, planeID):
        ecNum = eca.PutSize()

        x = np.array([0])
        xvalKey = ""
        if planeID == ViewMainData.SLICE_XY:
            minx = min(self.rangex)
            maxx = max(self.rangex)
            minx -= self.basicSliceAx1bin
            maxx += self.basicSliceAx1bin
            x  = np.arange(minx, maxx, self.basicSliceAx1bin)
            xvalKey = self.ax1Key
        elif planeID == ViewMainData.SLICE_YZ:
            minx = min(self.rangey)
            maxx = max(self.rangey)
            minx -= self.basicSliceAx2bin
            maxx += self.basicSliceAx2bin
            x  = np.arange(minx, maxx, self.basicSliceAx2bin)
            xvalKey = self.ax2Key
        elif planeID == ViewMainData.SLICE_ZX:
            minx = min(self.rangex)
            maxx = max(self.rangex)
            minx -= self.basicSliceAx1bin
            maxx += self.basicSliceAx1bin
            x  = np.arange(minx, maxx, self.basicSliceAx1bin)
            xvalKey = self.ax1Key

        if ecNum < len(x):
           x = x[0:ecNum]  # OLD...

        xlen = len(x)
        ec = eca.Put(0)
        y  = ec.PutXList()
        ylen = len(y)

        xval = eca.PutHeader().PutDoubleVector(str(xvalKey)) #[inamura 180422]
        xval_list = Manyo.DoubleVectorToList(xval)

        x = xval_list
#        print(xval_list)
#        print eca.PutHeader().Dump()
#        print ec.PutHeader().Dump()

        if planeID == ViewMainData.SLICE_XY:
            self.data_xy = np.zeros((xlen, ylen))
            self.data_xy_range = [min(x), max(x), min(y), max(y)]
            self.data_xy_pos = pos
        elif planeID == ViewMainData.SLICE_YZ:
            self.data_yz = np.zeros((xlen, ylen))
            self.data_yz_range = [min(x), max(x), min(y), max(y)]
            self.data_yz_pos = pos
        elif planeID == ViewMainData.SLICE_ZX:
#           self.data_zx = np.zeros((xlen, ylen))
#           variable x means z, variable y means x.
#           data_zx[X][Y]: [X] have to be x and [y] have to be z:
            self.data_zx = np.zeros((xlen, ylen))
            self.data_zx_range = [min(x), max(x), min(y), max(y)]
            self.data_zx_pos = pos

        tmpMin = 9999.0
        tmpMax = -9999.0
        self.intensityMin = tmpMin
        self.intensityMax = tmpMax

        for xid in range(xlen):
           ec = eca.Put(xid)
           intensity = ec.PutYList()
           #tmpMin = min(intensity)
           #tmpMax = max(intensity)

           for yid in range(len(intensity)):
              val = intensity[yid]
              #if val > 1.0: # 1e+100 ... Invalid Value ???
              if val > 1.0e+30:
                  val = 1e-16

              if val < tmpMin:
                  tmpMin = val
              elif tmpMax < val:
                  if val <= 1.0:
                      tmpMax = val
              if planeID == ViewMainData.SLICE_XY:
                  self.data_xy[xid,yid] = val
              elif planeID == ViewMainData.SLICE_YZ:
                  self.data_yz[xid,yid] = val
              elif planeID == ViewMainData.SLICE_ZX:
#                  self.data_zx[xid,yid] = val
                  self.data_zx[xid,yid] = val

           if tmpMin < self.intensityMin:
               self.intensityMin = tmpMin
           if self.intensityMax < tmpMax:
               self.intensityMax = tmpMax

        if self.viewSubData.autoScale == True:
            self.viewSubData.scalarRangeMin = self.intensityMin
            self.viewSubData.scalarRangeMax = self.intensityMax
            self.update_autoscale_signal.emit()

